unit Ucalc;

interface
var InputOpened, OneLayer, TwoLayer, ThreeLayer, DrainSystem,
    Result, TooDeep, HydrPaste, Change1, Change2, Change3,
    Change4, Change5, Checked, CalcDone, NoCount1, NoCount2,
    NoCount3, NoCount4, NoCount5, Impr                          : boolean;
    Title1, Title2, InitDir, FileName, OutFileName, InFileName  : string;
    NrOfSteps, StepLength, NrOfData, PrevIndex, NrOfLayers,
    Count1, Count2, Count3, Count4, Count5                      : integer;
    Rain, Irri, Evap, Evap1, Evap2, Evap3, PercF, CapF, EPAf,
    SaltRoot1M, SaltRoot2M, SaltRoot3M, SaltTransM, SaltAquifM,
    DepthWT, DRCf, SaltLay1, SaltLay2, SaltLay3, SaltTR, SaltAQ,
    DWTdat, Irrig, Stor, StorTot                       : array of real;
    Perc1, Perc2, Perc3, Cap1, Cap2, Cap3, Cap4        : array of array of real;
    RootZone, Layer1, Layer2, Layer3, Trans, Aquif,
    TotPor1, TotPor2, TotPor3, TotPorTR, TotPorAQ,
    EffPor1, EffPor2, EffPor3, EffPorTR, EffPorAQ, EffPorosity,
    LeachEff1, LeachEff2, LeachEff3, LeachEffTR, LeachEffAQ,
    LeachEff, DrainDepth, DrainQH1, DrainQH2, DrainQH1F, DRCk, DWTi,
    SaltIrrig, InitSaltRoot1, InitSaltRoot2, InitSaltRoot3,
    InitSaltTrans, InitSaltAquif, InitSaltR, InitSaltT, StorMax,
    InitSalt2, InitSalt3, SumSalt, SaltLayer1, SaltLayer2, SaltLayer3,
    SaltTrans, SaltAquif, SaltLayer1i, SaltLayer2i, SaltLayer3i,
    Salinity, SaltChange, SaltIn, SaltOut, AveSalt, SaltCap, SaltCap2,
    SaltCap3, InitDepthWT, GrwtInf, GrwtOut, GrwtDown, GrwtUp,
    ZZ, CDW, CDWR, ES, EPA, CAP, EPAT, CAPT, HCR, Moisture,
    DRA, DRB, DRC, DRAT, DRBT, DRCT, PERC, PERCT, IrrEff,
    SalInf, WaterDepthRo, WaterDepthTR, WaterDepthAQ,
    WaterDepth1, WaterDepth2, WaterDepth3, CritDepth,
//the waterdepths can be reduced to one only
    EPA1, EPA2, EPA3,
    AveDepth, TotDepth, Ratio1, Ratio2, Ratio3, Diff, FinalDiff, SumDiffSq,
    Ave, SumAvSq, Expl1, Expl2, Expl3, ExplTR, ExplAQ, Divider,
    LeachEff01, LeachEff02, LeachEff03, LeachEff0TR, LeachEff0AQ : real;
    Irritot,Raintot,Evaptot,Perctot,Draintot,Balancetot : real;  {for checking}
    OutFile : textfile;

    function  Question(const Query: string): boolean;
    function  StringToFloat(Str : string) : double;
    procedure SaveData;
    procedure Calculations;
    procedure WriteHeader;
    procedure ReadInput;
    procedure WaterBalance;
    procedure FinalBalance;
    procedure OneLayerRootzone;
    procedure OneLayerTranszone;
    procedure OneLayerAquifer;
    procedure TwoLayer1stPart;
    procedure TwoLayer2ndPart;
    procedure TwoLayerTrans;
    procedure TwoLayerAquif;
    procedure ThreeLayer1stPart;
    procedure ThreeLayer2ndPart;
    procedure ThreeLayer3rdPart;
    procedure ThreeLayerTrans;
    procedure ThreeLayerAquif;
    procedure SaltBalance;
    procedure WriteOutput;
    procedure WriteSaltTable;
    procedure WriteHydrTable;
    procedure ReadOutData;
    procedure ReadInpData;

var LEff1, LEff2, LEff3, LEff4, LEff5      : real;
    Optim1, Optim2, Optim3, Optim4, Optim5, Refine : boolean;

implementation

uses SysUtils, Dialogs, Controls, UMain;


function Question(const Query: string): boolean;
begin
  Result := MessageDlg(Query, mtConfirmation, [mbYes, mbNo], 0) = mrYes;
end;


function StringToFloat(Str : string) : double;
{accepts comma or point for decimal separator}
begin
  try
    Result := StrToFloat(Str);
  except
    on EConvertError do
    begin
      if DecimalSeparator = ',' then
        DecimalSeparator := '.';
      Result := StrToFloat(Str);
    end;
  end;
end; {TForm1.StringToFloat}


procedure SaveData;
var DataFile : textfile;
    i        : integer;
    AStr, Bstr, Cstr, Dstr, Estr : string;
begin
  AssignFile (DataFile,FileName);
  rewrite (DataFile);
  writeln (DataFile,' PROGRAM LeachMod INPUT DATA');
  writeln (DataFile,' Name of this file        : ',FileName);
  writeln (DataFile,' First title              : ',Title1);
  writeln (DataFile,' Second title             : ',Title2);
  writeln (DataFile,'    General data');
  if NrOfSteps<>0 then
  begin
     writeln (DataFile,' Nunber of time steps     : ',NrOfSteps);
     NrOfData:=NrOfSteps;
  end else
  begin
     writeln (DataFile,' Nunber of time steps     : ',NrOfData);
     NrOfSteps:=NrOfData;
  end;
  writeln (DataFile,' Step length (days)       : ',StepLength);
  writeln (DataFile,' Nr of root zone layers   : ',NrOfLayers);
  writeln (DataFile,'    Drainage data');
  if DrainSystem then
     writeln (DataFile,' Drainage system present  : yes')
  else
     writeln (DataFile,' Drainage system present  : no');
  if DrainSystem then
     writeln (DataFile,' Depth subsurf. drains(m) : ',DrainDepth:7:2)
  else
     writeln (DataFile,' Depth subsurf. drains(m)  : n.a.');
  if DrainSystem then
     writeln (DataFile,' Drain. react. fact. QH1  : ',DrainQH1:8:4)
  else
     writeln (DataFile,' Drain. react. fact. QH1  : n.a.');
  if DrainSystem then
     writeln (DataFile,' Drain. react. fact. QH2  : ',DrainQH2:8:4)
  else
     writeln (DataFile,' Drain. react. fact. QH2  : n.a.');
  writeln (DataFile,'    Initial data soil and water');
  writeln (DataFile,' Salt conc. irr. water (dS/m)        : ',SaltIrrig:8:1);
  if OneLayer then
    writeln (DataFile,' Number of layers in root zone       :     1');
  if TwoLayer then
    writeln (DataFile,' Number of layers in root zone       :     2');
  if ThreeLayer then
    writeln (DataFile,' Number of layers in root zone       :     3');
  if OneLayer then
  writeln (DataFile,' Init. salt conc. root zone (dS/m)   : ',
                    InitSaltRoot1:8:1);
  if TwoLayer then
  begin
    writeln (DataFile,' Init. salt conc. 1st layer (dS/m)   :',
    InitSaltRoot1:8:1);
    writeln (DataFile,' Init. salt conc. 2nd layer (dS/m)   :',
    InitSaltRoot2:8:1);
  end;
  if ThreeLayer then
  begin
    writeln (DataFile,' Init. salt conc. 1st layer (dS/m)   :',
    InitSaltRoot1:8:1);
    writeln (DataFile,' Init. salt conc. 2nd layer (dS/m)   :',
    InitSaltRoot2:8:1);
    writeln (DataFile,' Init. salt conc. 3rd layer (dS/m)   :',
    InitSaltRoot3:8:1);
  end;
 writeln (DataFile, ' Init. salt conc. trans. zone (dS/m) : ',
  InitSaltTrans:7:1);
 writeln (DataFile, ' Init. salt conc. aquifer (dS/m)     : ',
  InitSaltAquif:7:1);
  writeln (DataFile,' Initial depth water table (m)       : ',InitDepthWT:7:2);
  writeln (DataFile,' Critical depth water table (m)      : ',CritDepth:7:2);
  writeln (DataFile,' Initial soil moisture content (%)   : ',Moisture:7:2);
  writeln (DataFile,' Irrigation efficiency (%)           : ',IrrEff:7:1);
  writeln (DataFile,' Inflow of groundwater (mm/day)      : ',GrwtInf:7:1);
  writeln (DataFile,' Outflow of groundwater (mm/day)     : ',GrwtOut:7:1);
  writeln (DataFile,' Salinity groundwater inflow (dS/m)  : ',SalInf:7:1);
  writeln(DataFile);
  writeln (DataFile,' Hydrological data');
  writeln (DataFile,' Time Step     Rainfall      Irrigation    Evaporation');
  writeln (DataFile,'             mm/time step   mm/time step   mm/time step');
  for i:=1 to NrOfSteps do
     writeln (DataFile,i:10,Rain[i]:13:2,Irri[i]:16:2,Evap[i]:15:2);
  writeln(DataFile);
  if OneLayer then
  begin
    writeln (DataFile,' Soil properties, root zone has 1 layer');
    writeln (DataFile,'             Thickness (m)');
    writeln (DataFile,' - - - - - - - - - - - - - - - - - - -');
    writeln (DataFile,' Root zone   Transition zone   Aquifer');
    writeln (DataFile,  Layer1:10:2,Trans:18:2,Aquif:10:2);
    writeln (DataFile);
    writeln(DataFile,' Other soil properties in %');
    writeln (DataFile,'              Total porosity  Effective porosity'+
                      '  Leaching efficiency');
    writeln (Datafile,' Root zone   ',TotPor1:15:0,EffPor1:20:0,
                        LeachEff1:21:0);
    writeln (Datafile,' Trans. zone ',TotPorTR:15:0,EffPorTR:20:0,
                        LeachEffTR:21:0);
    writeln (Datafile,' Aquifer     ',TotPorAQ:15:0,EffPorAQ:20:0,
                        LeachEffAQ:21:0);
  end;
  if TwoLayer then
  begin
    writeln (DataFile,' Soil properties, root zone has 2 layers');
    writeln (DataFile,'                   Thickness (m) ');
    writeln (DataFile,' - - - - - - - - - - - - - - - - - - - - - -');
    writeln (DataFile,'    Root zone   ');
    writeln (DataFile,' - - - - - - - -   Transition zone   Aquifer');
    writeln (DataFile,' Layer1    Layer2');
    writeln (DataFile,  Layer1:7:2,Layer2:10:2,Trans:17:2,Aquif:10:2);
    writeln (DataFile);
    writeln(DataFile,' Other soil properties in %');
    writeln (DataFile,'              Total porosity  Effective porosity'+
                      '  Leaching efficiency');
    writeln (Datafile,' 1st layer   ',TotPor1:15:0,EffPor1:20:0,
                        LeachEff1:21:0);
    writeln (Datafile,' 2nd layer   ',TotPor2:15:0,EffPor2:20:0,
                        LeachEff2:21:0);
    writeln (Datafile,' Trans. zone ',TotPorTR:15:0,EffPorTR:20:0,
                        LeachEffTR:21:0);
    writeln (Datafile,' Aquifer     ',TotPorAQ:15:0,EffPorAQ:20:0,
                        LeachEffAQ:21:0);
  end;
  if ThreeLayer then
  begin
    writeln (DataFile,' Soil properties, root zone has 3 layers');
    writeln (DataFile,'                      Thickness (m)');
   writeln (DataFile,' - - - - - - - - - - - - - - - - - - - - - - - - - - - ');
   writeln (DataFile,'         Root zone   ');
   writeln (DataFile,' - - - - - - - - - - - - - -  Transition zone   Aquifer');
    writeln (DataFile,' Layer1    Layer2    Layer3');
  writeln (DataFile,  Layer1:7:2,Layer2:10:2,Layer3:10:2,Trans:17:2,Aquif:10:2);
    writeln (DataFile);
    writeln(DataFile,' Other soil properties in %');
    writeln (DataFile,'              Total porosity  Effective porosity'+
                      '  Leaching efficiency');
    writeln (Datafile,' 1st layer   ',TotPor1:15:0,EffPor1:20:0,
                        LeachEff1:21:0);
    writeln (Datafile,' 2nd layer   ',TotPor2:15:0,EffPor2:20:0,
                        LeachEff2:21:0);
    writeln (Datafile,' 3rd layer   ',TotPor3:15:0,EffPor3:20:0,
                        LeachEff3:21:0);
    writeln (Datafile,' Trans. zone ',TotPorTR:15:0,EffPorTR:20:0,
                        LeachEffTR:21:0);
    writeln (Datafile,' Aquifer     ',TotPorAQ:15:0,EffPorAQ:20:0,
                        LeachEffAQ:21:0);
  end;
  writeln (Datafile);
  if OneLayer then
  begin
    writeln (DataFile,' Measured soil moisture salinity data ECsm in dS/m,'+
                      ' root zone has 1 layer');
    writeln (DataFile,' Time step   Root zone   Transition zone   Aquifer');
    for i:=1 to NrOfSteps do
    begin
      if (SaltRoot1M[i]=-1) or (SaltRoot1M[i]=0) then Astr:='          -'
      else Astr:=format('%12.2f',[SaltRoot1M[i]]);
      if (SaltTransM[i]=-1) or (SaltTRansM[i]=0) then Bstr:='                -'
      else Bstr:=format('%18.2f',[SaltTransM[i]]);
      if (SaltAquifM[i]=-1) or (SaltAquifM[i]=0) then Cstr:='        -'
      else Cstr:=format('%10.2f',[SaltAquifM[i]]);
      writeln (Datafile,i:10,Astr,Bstr,Cstr);
    end;
  end;
  if TwoLayer then
  begin
    writeln (DataFile,' Measured soil moisture salinity data ECsm in dS/m,'+
                      ' root zone has 2 layers');
    writeln (DataFile,'              Root zone   ');
    writeln (DataFile,' Time Step  - - - - - - - -   Transition zone   Aquifer');
    writeln (DataFile,'            Layer1   Layer2');
    writeln (DataFile,' - - - - - - - - - - - - - -');
    for i:=1 to NrOfSteps do
    begin
      if (SaltRoot1M[i]=-1) or (SaltRoot1M[i]=0) then Astr:='       -'
      else Astr:=format('%8.2f',[SaltRoot1M[i]]);
      if (SaltRoot2M[i]=-1) or (SaltRoot2M[i]=0) then Dstr:='        -'
      else Dstr:=format('%9.2f',[SaltRoot2M[i]]);
      if (SaltTransM[i]=-1) or (SaltTransM[i]=0) then Bstr:='                 -'
      else Bstr:=format('%18.2f',[SaltTransM[i]]);
      if (SaltAquifM[i]=-1) or (SaltAquifM[i]=0) then Cstr:='        -'
      else Cstr:=format('%10.2f',[SaltAquifM[i]]);
      writeln (Datafile,i:10,Astr,Dstr,Bstr,Cstr);
    end;
  end;
  if ThreeLayer then
  begin
    writeln (DataFile,' Measured soil moisture salinity data ECsm in dS/m,'+
                      ' root zone has 2 layers');
    writeln (DataFile,'                   Root zone   ');
    writeln (DataFile,' Time Step  - - - - - - - - - - -   Transition zone'+
                      '   Aquifer');
    writeln (DataFile,'            Layer1  Layer2  Layer3');
    writeln (DataFile,' - - - - - - - - - - - - - - - - -');
    for i:=1 to NrOfSteps do
    begin
      if (SaltRoot1M[i]=-1) or (SaltRoot1M[i]=0) then Astr:='       -'
      else Astr:=format('%8.2f',[SaltRoot1M[i]]);
      if (SaltRoot2M[i]=-1) or (SaltRoot2M[i]=0) then Dstr:='       -'
      else Dstr:=format('%8.2f',[SaltRoot2M[i]]);
      if (SaltRoot3M[i]=-1) or (SaltRoot3M[i]=0) then Estr:='       -'
      else Estr:=format('%8.2f',[SaltRoot3M[i]]);
      if (SaltTransM[i]=-1) or (SaltTRansM[i]=0) then Bstr:='                -'
      else Bstr:=format('%18.2f',[SaltTransM[i]]);
      if (SaltAquifM[i]=-1) or (SaltAquifM[i]=0)  then Cstr:='        -'
      else Cstr:=format('%10.2f',[SaltAquifM[i]]);
      writeln (Datafile,i:10,Astr,Dstr,Estr,Bstr,Cstr);
    end;
  end;
  closefile (DataFile);

  setlength (Irrig,NrOfSteps+1);
  for i:=1 to NrOfSteps do Irrig[i]:=Irri[i];

  if StepLength=1 then for i:=1 to NrOfSteps do
  begin
    if i<NrOfSteps-1 then
       if (Irri[i]>0) and (Irri[i+1]<=0.01) then
       begin
         Irrig[i]:=Irri[i]/2;
         Irrig[i+1]:=Irri[i]/2;
       end;
    if i<NrOfSteps-2 then
       if (Irri[i]>0) and (Irri[i+1]<=0.01) and (Irri[i+2]<0.01) then
       begin
         Irrig[i]:=Irri[i]/3;
         Irrig[i+1]:=Irri[i]/3;
         Irrig[i+2]:=Irri[i]/3;
       end;
    if i<NrOfSteps-3 then
       if (Irri[i]>0) and (Irri[i+1]<=0.01) and (Irri[i+2]<0.01)
       and (Irri[i+3]<0.01) then
       begin
         Irrig[i]:=Irri[i]/4;
         Irrig[i+1]:=Irri[i]/4;
         Irrig[i+2]:=Irri[i]/4;
         Irrig[i+3]:=Irri[i]/4;
       end;
    if i<NrOfSteps-4 then
       if (Irri[i]>0) and (Irri[i+1]<=0.01) and (Irri[i+2]<0.01)
       and (Irri[i+3]<0.01) and (Irri[i+4]<0.01) then
       begin
         Irrig[i]:=Irri[i]/5;
         Irrig[i+1]:=Irri[i]/5;
         Irrig[i+2]:=Irri[i]/5;
         Irrig[i+3]:=Irri[i]/5;
         Irrig[i+4]:=Irri[i]/5;
       end;
  end;

end; {SaveData}


procedure Calculations;
var i, j, k : integer;
    TmpSal : real;
begin

//Initial settings
  TooDeep:=false;
  Change1:=false;
  Change2:=false;
  Change3:=false;
  Change4:=false;
  Change5:=false;
  SaltLayer1:=InitSaltRoot1;
  SaltLayer2:=InitSaltRoot2;
  SaltLayer3:=InitsaltRoot3;
  SaltTrans:=InitSaltTrans;
  SaltCap:=InitSaltTrans;
  SaltAquif:=InitSaltAquif;
  SaltAquif:=InitSaltAquif;
  TotPor1:=TotPor1/100;
  TotPor2:=TotPor2/100;
  TotPor3:=TotPor3/100;
  TotPorTR:=TotPorTR/100;
  TotPorAQ:=TotPorAQ/100;
  EffPor1:=EffPor1/100;
  EffPor2:=EffPor2/100;
  EffPor3:=EffPor3/100;
  EffPorTR:=EffPorTR/100;
  EffPorAQ:=EffPorAQ/100;
  LeachEff1:=LeachEff1/100;
  LeachEff2:=LeachEff2/100;
  LeachEff3:=LeachEff3/100;
  LeachEffTR:=LeachEffTR/100;
  LeachEffAQ:=LeachEffAQ/100;
  IrrEff:=IrrEff/100;
  LeachEff01:=LeachEff1;
  LeachEff02:=LeachEff2;
  LeachEff03:=LeachEff3;
  LeachEff0TR:=LeachEffTR;
  LeachEff0AQ:=LeachEffAQ;
  if OneLayer then RootZone:=Layer1;
  if TwoLayer then Rootzone:=Layer1+Layer2;
  if ThreeLayer then Rootzone:=Layer1+Layer2+Layer3;

  CDW:=CritDepth;                             {critical depth WT for cap. rise}

//Counting number of observed salinities
  Count1:=0;
  Count2:=0;
  Count3:=0;
  Count4:=0;
  Count5:=0;
  for i:=1 to NrOfSteps do
  begin
    if SaltRoot1M[i]>0 then Count1:=Count1+1;
    if TwoLayer or ThreeLayer then
       if SaltRoot2M[i]>0 then Count2:=Count1+1;
    if ThreeLayer then
       if SaltRoot3M[i]>0 then Count3:=Count3+1;
    if SaltTransM[i]>0 then Count4:=Count1+1;
    if SaltAquifM[i]>0 then Count5:=Count1+1;
  end;

//Auxiliary parameters
  setlength (DepthWT,NrOfSteps+1);
  DepthWT[0]:=InitDepthWT;
  setlength (DWTdat,NrOfSteps+1);
  setlength (DRCf,NrOfSteps+2);
  setlength (PercF,NrOfSteps+1);
  setlength (CapF,NrOfSteps+1);
  setlength (EPAf,NrOfSteps+1);
  setlength (Stor,NrOfSteps+1);
  setlength (StorTot,NrOfSteps+1);
  setlength (SaltLay1,NrOfSteps+1);
  setlength (SaltLay2,NrOfSteps+1);
  setlength (SaltLay3,NrOfSteps+1);
  setlength (SaltTR,NrOfSteps+1);
  setlength (SaltAQ,NrOfSteps+1);
//Two dimensional arrays, assigning ratios
  setlength (Evap1,NrOfSteps+1);
  setlength (Perc1,NrOfSteps+1,StepLength+1);
  setlength (Cap1,NrOfSteps+1,StepLength+1);
  if TwoLayer or ThreeLayer then
  begin
    setlength (Evap2,NrOfSteps+1);
    setlength (Perc2,NrOfSteps+1,StepLength+1);
    setlength (Cap2,NrOfSteps+1,StepLength+1);
    setlength (Cap3,NrOfSteps+1,StepLength+1);
    TotDepth:=(Layer1+Layer2);
    AveDepth:=TotDepth/2;
    Ratio1:=(Layer1-AveDepth)/TotDepth;
    Ratio2:=(Layer2-AveDepth)/TotDepth;
  end;
  if ThreeLayer then
  begin
    setlength (Evap3,NrOfSteps+1);
    setlength (Perc3,NrOfSteps+1,StepLength+1);
    setlength (Cap4,NrOfSteps+1,StepLength+1);
    TotDepth:=(Layer1+Layer2+Layer3);
    AveDepth:=TotDepth/3;
    Ratio1:=(Layer1-AveDepth)/TotDepth;
    Ratio2:=(Layer2-AveDepth)/TotDepth;
    Ratio3:=(Layer3-AveDepth)/TotDepth;
  end;

//Distributing Evaporation over the layers
  if OneLayer then
     for i:=1 to NrOfSteps do Evap1[i]:=Evap[i];
  if TwoLayer then
  begin
    for i:=1 to NrOfSteps do Evap1[i]:=(2/3+Ratio1)*Evap[i];
    for i:=1 to NrOfSteps do Evap2[i]:=(1/3+Ratio2)*Evap[i];
  end;
  if ThreeLayer then
  begin
    for i:=1 to NrOfSteps do Evap1[i]:=(0.5+Ratio1)*Evap[i];
    for i:=1 to NrOfSteps do Evap2[i]:=(1/3+Ratio2)*Evap[i];
    for i:=1 to NrOfSteps do Evap3[i]:=(1/6+Ratio3)*Evap[i];
  end;

//Reducing hydrological values per time step to daily values
  for i:=1 to NrOfSteps do
  begin
    Rain[i]:=Rain[i]/StepLength;                                        {in mm}
    Irri[i]:=Irri[i]/StepLength;                                        {in mm}
    Irrig[i]:=Irrig[i]/StepLength;                                        {in mm}
    Evap[i]:=Evap[i]/StepLength;
    Evap1[i]:=Evap1[i]/StepLength;                                      {in mm}
    if TwoLayer or ThreeLayer then
       Evap2[i]:=Evap2[i]/StepLength;                                   {in mm}
    if ThreeLayer then
       Evap3[i]:=Evap3[i]/StepLength;                                   {in mm}
  end;

//Upward/downward ground water movement
  GrwtDown:=(GrwtOut-GrwtInf);
  if GrwtDown<0 then GrwtDown:=0;
  GrwtUp:=(GrwtInf-GrwtOut);
  if GrwtUp<0 then GrwtUp:=0;

//Amount of water stored per layer
  WaterDepth1:=Layer1*TotPor1*1000;           {rootzone, layer 1, from m to mm}
  if WaterDepth1<0.001 then WaterDepth1:=0.001;
  WaterDepth2:=Layer2*TotPor2*1000;           {rootzone, layer 2, from m to mm}
  if WaterDepth2<0.001 then WaterDepth2:=0.001;
  WaterDepth3:=Layer3*TotPor3*1000;           {rootzone, layer 3, from m to mm}
  if WaterDepth3<0.001 then WaterDepth3:=0.001;
  WaterDepthRO:=Layer1*TotPor1*1000;                   {rootzone, from m to mm}
  if WaterDepthRO<0.001 then WaterDepthRO:=0.001;
  WaterDepthTR:=Trans*TotPorTR*1000;                {trans. zone, from m to mm}
  if WaterDepthTR<0.001 then WaterDepthTR:=0.001;
  WaterDepthAQ:=Aquif*TotPorAQ*1000;                    {aquifer, from m to mm}
  if WaterDepthAQ<0.001 then WaterDepthAQ:=0.001;

  MainForm.ProgressBar1.Position:=2;
  sleep(100);
  Refine:=false;
  Result := Question('Leachmod can refine the water balance by selecting'+
                     ' a fitting drainage capacity. Do you wish to do the'+
                     ' refinement?');
  if Result then Refine:=true;
  if Refine then
  begin
    WaterBalance;
    FinalBalance;
  end else
  begin
    DrainQH1F:=DrainQH1;
    FinalBalance;
  end;
  MainForm.ProgressBar1.Position:=3;
  sleep(100);

//The following until they are taken up in the input file
  SaltLayer1i:=InitSaltRoot1;
  SaltLayer2i:=InitSaltRoot2;
  SaltLayer3i:=InitSaltRoot3;

  if (Count1>3) or (Count2>3) or (Count3>3) or (Count4>3) or (Count5>3) then
  Result := Question('There are one or more soil layers with enough data'+
                     ' on measured salinity to optimize the leaching'+
                     ' efficiency. Do you want to do the optimization?');
  if Result then
  BEGIN

//Optimizing leaching efficiency in different modes
  if OneLayer and (Count1>3) then OneLayerRootzone;
  if OneLayer and (Count4>3) then OneLayerTranszone;
  if OneLayer and (Count5>3) then OneLayerAquifer;
  if TwoLayer and (Count1>3) then TwoLayer1stPart;
  if TwoLayer and (Count2>3) then TwoLayer2ndPart;
  if TwoLayer and (Count4>3) then TwoLayerTrans;
  if TwoLayer and (Count5>3) then TwoLayerAquif;
  if ThreeLayer and (Count1>3) then ThreeLayer1stPart;
  if ThreeLayer and (Count2>3) then ThreeLayer2ndPart;
  if ThreeLayer and (Count3>3) then ThreeLayer3rdPart;
  if ThreeLayer and (Count4>3) then ThreeLayerTrans;
  if ThreeLayer and (Count5>3) then ThreeLayerAquif;
  if ThreeLayer and (Count4>3) then ThreeLayerTrans;
  if ThreeLayer and (Count5>3) then ThreeLayerAquif;
  MainForm.ProgressBar1.Position:=4;
  sleep(100);

  END; {IF RESULT THEN}

  MainForm.ProgressBar1.Position:=6;
  sleep(100);

  SaltLayer1:=SaltLayer1i;
  SaltLayer2:=SaltLayer2i;
  SaltLayer3:=SaltLayer3i;
  SaltTrans:=InitSaltTrans;
  SaltAquif:=InitSaltAquif;

  SaltBalance;
  MainForm.ProgressBar1.Position:=8;
  sleep(100);

//Preparing output
  if LeachEff01<>LeachEff1 then Change1:=true;
  if LeachEff02<>LeachEff2 then Change2:=true;
  if LeachEff03<>LeachEff3 then Change3:=true;
  if LeachEff0TR<>LeachEffTR then Change4:=true;
  if LeachEff0AQ<>LeachEffAQ then Change5:=true;

//Writing output
  WriteHeader;

//for checking
  Irritot:=0;
  Raintot:=0;
  Evaptot:=0;
  Perctot:=0;
  Draintot:=0;
  Balancetot:=0;
  DRCf[0]:=DRCf[1];
  MainForm.ProgressBar1.Position:=8;
  sleep(100);

  for i:=1 to NrOfSteps do
  begin
    Rain[i]:=Rain[i]*StepLength;
    Irri[i]:=Irri[i]*StepLength;
    Irrig[i]:=Irrig[i]*StepLength;
    Evap[i]:=Evap[i]*StepLength;
    if OneLayer then
       writeln (OutFile,i:9,format('%9.1f',[PercF[i]]),
                format('%9.1f',[CapF[i]]),format('%9.1f',[EPAf[i]]),
                format('%10.1f',[DRCf[i]]),format('%10.2f',[DepthWT[i]]),
                format('%10.2f',[SaltLay1[i]]),
                format('%13.2f',[SaltTR[i]]),format('%9.2f',[SaltAQ[i]]));
    if TwoLayer then
       writeln (OutFile,i:9,format('%9.1f',[PercF[i]]),
                format('%11.1f',[CapF[i]]),format('%9.1f',[EPAf[i]]),
                format('%10.1f',[DRCf[i]]),format('%8.2f',[DepthWT[i]]),
                format('%9.2f',[SaltLay1[i]]),format('%8.2f',[SaltLay2[i]]),
                format('%8.2f',[SaltTR[i]]),format('%10.2f',[SaltAQ[i]]));
    if ThreeLayer then
       writeln (OutFile,i:9,format('%9.1f',[PercF[i]]),
                format('%11.1f',[CapF[i]]),format('%9.1f',[EPAf[i]]),
                format('%10.1f',[DRCf[i]]),format('%8.2f',[DepthWT[i]]),
                format('%9.2f',[SaltLay1[i]]),
                format('%9.2f',[SaltLay2[i]]),format('%8.2f',[SaltLay3[i]]),
                format('%9.2f',[SaltTR[i]]),format('%10.2f',[SaltAQ[i]]));

//Checking balances
    Irritot:=Irritot+Irri[i];
    Raintot:=RainTot+Rain[i];
    Evaptot:=Evaptot+EPAf[i];
    Perctot:=Perctot+Percf[i];
    Draintot:=Draintot+DRCf[i];
    Balancetot:=Balancetot + Irri[i] + Rain[i] - EPAf[i] - DRCf[i];
  end;

//Restore values
  TotPor1:=TotPor1*100;
  TotPor2:=TotPor2*100;
  TotPor3:=TotPor3*100;
  TotPorTR:=TotPorTR*100;
  TotPorAQ:=TotPorAQ*100;
  EffPor1:=EffPor1*100;
  EffPor2:=EffPor2*100;
  EffPor3:=EffPor3*100;
  EffPorTR:=EffPorTR*100;
  EffPorAQ:=EffPorAQ*100;
  LeachEff1:=LeachEff01*100;
  LeachEff2:=LeachEff02*100;
  LeachEff3:=LeachEff03*100;
  LeachEffTR:=LeachEff0TR*100;
  LeachEffAQ:=LeachEff0AQ*100;
  IrrEff:=IrrEff*100;
  MainForm.ProgressBar1.Position:=9;
  sleep(100);

  closefile (OutFile);

end; {procedure Calculations}


procedure WriteHeader;
begin
  OutFileName:=changefileext (FileName,'.out');
  assignfile (OutFile,OutFileName);
  rewrite (OutFile);
  writeln (OutFile,'OUTPUT OF LeachMod');
  writeln (OutFile,'Name of this file :'+OutFileName);
  writeln (OutFile);
  writeln (OutFile,Title1);
  writeln (OutFile,Title2);
  writeln (OutFile,NrOfLayers:2,'   ',NrOfSteps:4,'    ',StepLength,
           '      (resp.:  nr. of root zone layers,    tot. nr. of time steps,',
                   '    length of time step in days)');
  if Refine and Impr then
     Writeln (OutFile, ' Drainage capacity QH1 refined :',DrainQH1F:8:3);
  if Refine and not Impr then
     Writeln (OutFile, ' Drainage capacity QH1 (given) :',DrainQH1F:8:3);
  if not NoCount1 and Change1 then
     Writeln (OutFile, ' Leaching efficiency layer 1   :',LeachEff1:8:2,
         '  (optimized),   Coeff. of determination (%) :',100*Expl1:5:0);
  if not NoCount1 and not Change1 then
     Writeln (OutFile, ' Leaching efficiency layer 1   :',LeachEff1:8:2,
         '  (given),       Coeff. of determination (%) :',100*Expl1:5:0);
  if NoCount1 then
     Writeln (OutFile, ' Leaching efficiency layer 1   :',LeachEff1:8:2,
         '  (given),       Coeff. of determination (%) :  n.a.');
  if TwoLayer or ThreeLayer then
  begin
    if not NoCount2 and Change2 then
       Writeln (OutFile, ' Leaching efficiency layer 2   :',LeachEff2:8:2,
           '  (optimized),   Coeff. of determination (%) :',100*Expl2:5:0);
    if not NoCount2 and not Change2 then
       Writeln (OutFile, ' Leaching efficiency layer 2   :',LeachEff2:8:2,
           '  (given),       Coeff. of determination (%) :',100*Expl2:5:0);
    if NoCount2 then
       Writeln (OutFile, ' Leaching efficiency layer 2   :',LeachEff2:8:2,
           '  (given),       Coeff. of determination (%) :  n.a.');
  end;
  if ThreeLayer then
  begin
    if not NoCount3 and Change3 then
       Writeln (OutFile, ' Leaching efficiency layer 3   :',LeachEff3:8:2,
           '  (optimized),   Coeff. of determination (%) :',100*Expl3:5:0);
    if not NoCount3 and not Change3 then
       Writeln (OutFile, ' Leaching efficiency layer 3   :',LeachEff3:8:2,
           '  (given),       Coeff. of determination (%) :',100*Expl3:5:0);
    if NoCount3 then
       Writeln (OutFile, ' Leaching efficiency layer 3   :',LeachEff3:8:2,
           '  (given),       Coeff. of determination (%) :  n.a.');
  end;
  if LeachEffTR>0 then
  begin
    if not NoCount4 and Change4 then
       Writeln (OutFile, ' Leaching eff. transit. zone   :',LeachEffTR:8:2,
           '  (optimized),   Coeff. of determination (%) :',100*ExplTR:5:0);
    if not NoCount4 and not Change4 then
       Writeln (OutFile, ' Leaching eff. transit. zone   :',LeachEffTR:8:2,
           '  (given),       Coeff. of determination (%) :',100*ExplTR:5:0);
    if NoCount4 then
       Writeln (OutFile, ' Leaching eff. transit. zone   :',LeachEffTR:8:2,
           '  (given),       Coeff. of determination (%) :  n.a.');
  end;
  if LeachEffAQ>0 then
  begin
    if not NoCount5 and Change5 then
       Writeln (OutFile, ' Leaching efficiency aquifer   :',LeachEffAQ:8:2,
           '  (optimized),   Coeff. of determination (%) :',100*ExplAQ:5:0);
    if not NoCount5 and not Change5 then
       Writeln (OutFile, ' Leaching efficiency aquifer   :',LeachEffAQ:8:2,
           '  (given),       Coeff. of determination (%) :',100*ExplAQ:5:0);
    if NoCount5 then
       Writeln (OutFile, ' Leaching efficiency aquifer   :',LeachEffAQ:8:2,
           '  (given),       Coeff. of determination (%) :  n.a.');
  end;

  writeln (OutFile,' Units:');
  writeln (OutFile,' Water flow in mm/time step,  Salinity in dS/m,  '+
                   ' Depth water table (DepthWT) in m');
  writeln (OutFile);

  if OneLayer then
  begin
    writeln (OutFile,'           Percol.  Cap. rise  Act. Evap  Drainage'+
    '                    Salinity (dS/m)');
    writeln (OutFile,' Time step - - - - - - - - - - - - - - - - - - - -'+
    ' DepthWT   - - - - - - - - - - - - - - - -');
    writeln (OutFile,'                      (mm/time step)              '+
    ' (m)        Rootzone  Trans. zone  Aquifer');
    writeln (OutFile,' - - - - - - - - - - - - - - -- - - - - - - - - - '+
    '- - - - - - - - - - - - - - - -');
  end;
  if TwoLayer then
  begin
    writeln (OutFile,'           Percol.  Cap. rise  Act. Evap  Drainage'+
    '                    Salinity (dS/m)');
    writeln (OutFile,' Time step - - - - - - - - - - - - - - - - - - - -'+
    ' DepthWT   - - - - - - - - - - - - - - - -');
    writeln (OutFile,'                      (mm/time step)              '+
    ' (m)        Rootzone ');
    writeln (OutFile,'                                                  '+
    '          - - - - - - -  Transit.  Aquifer');
    writeln (OutFile,'                                                  '+
    '          Layer1  Layer2   zone');
    writeln (OutFile,' - - - - - - - - - - - - - - -- - - - - - - - - - '+
    '- - - - - - - - - - - - - - - - - - - - -');
  end;
  if ThreeLayer then
  begin
    writeln (OutFile,'           Percol.  Cap. rise  Act. Evap  Drainage'+
    '                         Salinity (dS/m)');
    writeln (OutFile,' Time step - - - - - - - - - - - - - - - - - - - -'+
    ' DepthWT   - - - - - - - - - - - - - - - - - - - -');
    writeln (OutFile,'                      (mm/time step)               (m)'+
    '              Rootzone ');
    writeln (OutFile,'                                                  '+
    '          - - - - - - - - - - - - Transit.  Aquifer');
    writeln (OutFile,'                                                  '+
    '          Layer1  Layer2  Layer3   zone');
    writeln (OutFile,' - - - - - - - - - - - - - - -- - - - - - - - - - '+
    '- - - - - - - - - - - - - - - - - - - - - - - - - -');
  end;
end; {procedure WriteHeader}



procedure ReadInput;
var DataFile : textfile;
    DrSyst : string[4];
    Str8, Str8A, Str8B : string[8];
    Str9   : string[9];
    Str10, Str10A, Str10B  : string[10];
    Str15A, Str15B, Str15C  : string[15];
    Str18, Str18A,Str18B,Str18C  : string[18];
    Str12  : string[12];
    Str20  : string[20];
    Str28  : string[28];
    Str38  : string[38];
    i      : integer;
    NewVersion : boolean;

begin

  AssignFile (DataFile,FileName);
  reset (DataFile);
  readln(DataFile);
  readln(DataFile);
  readln(DataFile,Str28,Title1);
  MainForm.Tit1Edit.Text:=Title1;
  readln(DataFile,Str28,Title2);
  MainForm.Tit2Edit.Text:=Title2;
  readln(DataFile);
  readln(DataFile,Str28,NrOfSteps);
  MainForm.NrTstEdit.Text:=inttostr(NrOfSteps);
  NrOfData:=NrOfSteps;
  readln(DataFile,Str28,StepLength);
  if StepLength=360 then MainForm.TstCombo.ItemIndex:=0
  else if StepLength=30 then MainForm.TstCombo.ItemIndex:=1
  else if StepLength=7 then MainForm.TstCombo.ItemIndex:=2
  else if StepLength=1 then MainForm.TstCombo.ItemIndex:=3;
  readln(DataFile,Str28,NrOfLayers);
  PrevIndex:=NrOfLayers-1;
//Reading drainage parameters
  readln (DataFile);
  readln(DataFile,Str28,DrSyst);
  if DrSyst='yes' then
  begin
    MainForm.DrainPresentCombo.ItemIndex:=1;
    MainForm.PrevDrInd:=1;
    DrainSystem:=true;
    MainForm.DrainDepthLabel.Enabled:=true;
    MainForm.DrainDepthEdit.Enabled:=true;
    MainForm.DrainQH1Label.Enabled:=true;
    MainForm.DrainQH1Edit.Enabled:=true;
    MainForm.DrainQH2Label.Enabled:=true;
    MainForm.DrainQH2Edit.Enabled:=true;
  end else
  begin
    MainForm.DrainPresentCombo.ItemIndex:=0;
    MainForm.PrevDrInd:=0;
    DrainSystem:=false;
    MainForm.DrainDepthLabel.Enabled:=false;
    MainForm.DrainDepthEdit.Enabled:=false;
    MainForm.DrainDepthEdit.Text:=' ';
    MainForm.DrainQH1Label.Enabled:=false;
    MainForm.DrainQH1Edit.Enabled:=false;
    MainForm.DrainQH1Edit.Text:=' ';
    MainForm.DrainQH2Label.Enabled:=false;
    MainForm.DrainQH2Edit.Enabled:=false;
    MainForm.DrainQH2Edit.Text:=' ';
  end;
  if DrainSystem then
  begin
    readln(DataFile,Str28,Str10);
    DrainDepth:=StringToFloat(Str10);
    MainForm.DrainDepthEdit.Text:=floattostr(DrainDepth);
    readln(DataFile,Str28,Str10);
    DrainQH1:=StringToFloat(Str10);
    MainForm.DrainQH1Edit.Text:=floattostr(DrainQH1);
    readln(DataFile,Str28,Str10);
    DrainQH2:=StringToFloat(Str10);
    MainForm.DrainQH2Edit.Text:=floattostr(DrainQH2);
  end else
  begin
    readln(DataFile);
    readln(DataFile);
    readln(DataFile);
  end;
  readln(DataFile, Str20);
  if Str20='    Initial data' then NewVersion := false
  else NewVersion := true;

//Reading irrigation salinity
  readln(DataFile,Str38,Str10);
  SaltIrrig:=StringToFloat(Str10);
    MainForm.SaltIrrigEdit.Text:=floattostr(SaltIrrig);

//Reading number of layers in the rootzone
  readln(DataFile,Str38,NrOfLayers);
    if NrOfLayers=1 then
    begin
      MainForm.LayersCombo.ItemIndex:=0;
      OneLayer:=true;
      TwoLayer:=false;
      ThreeLayer:=false;
    end;
    if NrOfLayers=2 then
    begin
      MainForm.LayersCombo.ItemIndex:=1;
      OneLayer:=false;
      TwoLayer:=true;
      ThreeLayer:=false;
    end;
    if NrOfLayers=3 then
    begin
      MainForm.LayersCombo.ItemIndex:=2;
      OneLayer:=false;
      TwoLayer:=false;
      ThreeLayer:=true;
    end;

//Reading initial conditions
    readln(DataFile,Str38,Str10);
    InitSaltRoot1:=StringToFloat(Str10);
    if NewVersion then
    begin
      if Twolayer or ThreeLayer then
      begin
        readln(DataFile,Str38,Str10);
        InitSaltRoot2:=StringToFloat(Str10);
      end;
      if ThreeLayer then
      begin
        readln(DataFile,Str38,Str10);
        InitSaltRoot3:=StringToFloat(Str10);
      end;
    end else
    begin
      InitSaltRoot2:=InitSaltRoot1;
      InitSaltRoot3:=InitSaltRoot1;
    end;
    readln(DataFile,Str38,Str10);
    InitSaltTrans:=StringToFloat(Str10);
    readln(DataFile,Str38,Str10);
    InitSaltAquif:=StringToFloat(Str10);
    readln(DataFile,Str38,Str10);
    InitDepthWT:=StringToFloat(Str10);
    MainForm.WTdepthEdit.Text:=floattostr(InitDepthWT);
    readln(DataFile,Str38,Str10);
    CritDepth:=StringToFloat(Str10);
    MainForm.CritDepthEdit.Text:=floattostr(CritDepth);
    read(DataFile,Str18);
    if Str18=' Initial soil mois' then
    begin
      readln (DataFile,Str20,Str10);
      Moisture:=StringToFloat(Str10);
      readln(DataFile,Str38,Str10);
      IrrEff:=StringToFloat(Str10);
    end else
    begin
      readln (DataFile,Str20,Str10);
      IrrEff:=StringToFloat(Str10);
      Moisture:=50;
    end;
    MainForm.MoistEdit.Text:=floattostr(Moisture);
    MainForm.IrrEffEdit.Text:=floattostr(IrrEff);
    readln(DataFile,Str38,Str10);
    GrwtInf:=StringToFloat(Str10);
    MainForm.GrwtInfEdit.Text:=floattostr(GrwtInf);
    readln(DataFile,Str38,Str10);
    GrwtOut:=StringToFloat(Str10);
    MainForm.GrwtOutEdit.Text:=floattostr(GrwtOut);
    readln(DataFile,Str38,Str10);
  Try
    begin
      SalInf:=StringToFloat(Str10);
      MainForm.SalInfEdit.Text:=floattostr(SalInf);
      for i:=1 to 4 do readln(DataFile,Str28);
    end;
    Except on E: exception do
    begin
      MainForm.SalInfEdit.Text:='0';
      for i:=1 to 2 do readln(DataFile,Str28);
    end;
  end;
//Reading hydrological data
  setlength(Rain,NrOfSteps+1);
  setlength(Irri,NrOfSteps+1);
  setlength(Irrig,NrOfSteps+1);
  setlength(Stor,NrOfSteps+1);
  setlength(Evap,NrOfSteps+1);
  MainForm.HydrGrid.Cells[0,0]:='Time step';
  NewVersion:=false;
  for i:=1 to NrOfSteps do with MainForm do
  begin
    MainForm.HydrGrid.Cells[0,i]:=inttostr(i);
    readln(Datafile,Str10,Str15A,Str15B,Str15C);
    if Str10=' Hydrologi' then
    begin
      Showmessage ('This input file is of an extended version of LeachMod'+
                   ' called LeachModE and cannot be used here.'+
                   ' The program halts.');
      NewVersion:= true;
      exit;
    end;
    Rain[i]:=stringtofloat(str15A);
    Irri[i]:=stringtofloat(str15B);
    Evap[i]:=stringtofloat(str15C);
    HydrGrid.Cells[1,i]:=floattostr(Rain[i]);
    HydrGrid.Cells[2,i]:=floattostr(Irri[i]);
    HydrGrid.Cells[3,i]:=floattostr(Evap[i]);
    HydrGrid.RowCount:=NrOfSteps+1;
  end;

  for i:=1 to 5 do readln(DataFile,str12);
  if TwoLayer or ThreeLayer then for i:=1 to 2 do
     readln(DataFile, str12);
//Reading thicknesses
  if OneLayer then
  begin
    readln (DataFile,Str10,Str18,Str8);
    Layer1:=stringtofloat(Str10);
    Trans:=stringtofloat(Str18);
    Aquif:=stringtofloat(Str8);
//Make strings and remove blanks
    MainForm.Soil1Grid.Cells[0,0]:=floattostr(Layer1);
    MainForm.Soil1Grid.Cells[0,1]:=floattostr(Trans);
    MainForm.Soil1Grid.Cells[0,2]:=floattostr(Aquif);
  end;
  if TwoLayer then
  begin
    readln (DataFile,Str10,Str10A,Str18,Str8);
    Layer1:=stringtofloat(Str10);
    Layer2:=stringtofloat(Str10A);
    Trans:=stringtofloat(Str18);
    Aquif:=stringtofloat(Str8);
//Make strings and remove blanks
    MainForm.Soil2Grid.Cells[0,0]:=floattostr(Layer1);
    MainForm.Soil2Grid.Cells[0,1]:=floattostr(Layer2);
    MainForm.Soil2Grid.Cells[0,2]:=floattostr(Trans);
    MainForm.Soil2Grid.Cells[0,3]:=floattostr(Aquif);
  end;
  if ThreeLayer then
  begin
//    readln (DataFile,Layer1,Layer2,Layer3,Trans,Aquif);
    readln (DataFile,Str10,Str10A,Str10B,Str18,Str8);
    Layer1:=stringtofloat(Str10);
    Layer2:=stringtofloat(Str10A);
    Layer3:=stringtofloat(Str10B);
    Trans:=stringtofloat(Str18);
    Aquif:=stringtofloat(Str8);
//Make strings and remove blanks
    MainForm.Soil3Grid.Cells[0,0]:=floattostr(Layer1);
    MainForm.Soil3Grid.Cells[0,1]:=floattostr(Layer2);
    MainForm.Soil3Grid.Cells[0,2]:=floattostr(Layer3);
    MainForm.Soil3Grid.Cells[0,3]:=floattostr(Trans);
    MainForm.Soil3Grid.Cells[0,4]:=floattostr(Aquif);
  end;

  readln(DataFile,Str12);
  readln(DataFile,Str12);
  readln(DataFile,Str12);
  if NewVersion then readln (DataFile,Str12);

//Reading soil properties
  if OneLayer then
  begin
    readln(DataFile,str18,Str18A,Str18B,Str18C);
    TotPor1:=stringtofloat(Str18A);
    EffPor1:=stringtofloat(Str18B);
    LeachEff1:=stringtofloat(Str18C);
    readln(DataFile,str18,Str18A,Str18B,Str18C);
    TotPorTR:=stringtofloat(Str18A);
    EffPorTR:=stringtofloat(Str18B);
    LeachEffTR:=stringtofloat(Str18C);
    readln(DataFile,str18,Str18A,Str18B,Str18C);
    TotPorAQ:=stringtofloat(Str18A);
    EffPorAQ:=stringtofloat(Str18B);
    LeachEffAQ:=stringtofloat(Str18C);
    MainForm.Soil1Grid.Cells[1,0]:=floattostr(TotPor1);
    MainForm.Soil1Grid.Cells[1,1]:=floattostr(TotPorTR);
    MainForm.Soil1Grid.Cells[1,2]:=floattostr(TotPorAQ);
    MainForm.Soil1Grid.Cells[2,0]:=floattostr(EffPor1);
    MainForm.Soil1Grid.Cells[2,1]:=floattostr(EffPorTR);
    MainForm.Soil1Grid.Cells[2,2]:=floattostr(EffPorAQ);
    MainForm.Soil1Grid.Cells[3,0]:=floattostr(LeachEff1);
    MainForm.Soil1Grid.Cells[3,1]:=floattostr(LeachEffTR);
    MainForm.Soil1Grid.Cells[3,2]:=floattostr(LeachEffAQ);
    MainForm.Soil1Grid.Cells[4,0]:=floattostr(InitSaltRoot1);
    MainForm.Soil1Grid.Cells[4,1]:=floattostr(InitSaltTrans);
    MainForm.Soil1Grid.Cells[4,2]:=floattostr(InitSaltAquif);
  end;
  if TwoLayer then
  begin
    readln(DataFile,str18,Str18A,Str18B,Str18C);
    TotPor1:=stringtofloat(Str18A);
    EffPor1:=stringtofloat(Str18B);
    LeachEff1:=stringtofloat(Str18C);
    readln(DataFile,str18,Str18A,Str18B,Str18C);
    TotPor2:=stringtofloat(Str18A);
    EffPor2:=stringtofloat(Str18B);
    LeachEff2:=stringtofloat(Str18C);
    readln(DataFile,str18,Str18A,Str18B,Str18C);
    TotPorTR:=stringtofloat(Str18A);
    EffPorTR:=stringtofloat(Str18B);
    LeachEffTR:=stringtofloat(Str18C);
    readln(DataFile,str18,Str18A,Str18B,Str18C);
    TotPorAQ:=stringtofloat(Str18A);
    EffPorAQ:=stringtofloat(Str18B);
    LeachEffAQ:=stringtofloat(Str18C);
    MainForm.Soil2Grid.Cells[1,0]:=floattostr(TotPor1);
    MainForm.Soil2Grid.Cells[1,1]:=floattostr(TotPor2);
    MainForm.Soil2Grid.Cells[1,2]:=floattostr(TotPorTR);
    MainForm.Soil2Grid.Cells[1,3]:=floattostr(TotPorAQ);
    MainForm.Soil2Grid.Cells[2,0]:=floattostr(EffPor1);
    MainForm.Soil2Grid.Cells[2,1]:=floattostr(EffPor2);
    MainForm.Soil2Grid.Cells[2,2]:=floattostr(EffPorTR);
    MainForm.Soil2Grid.Cells[2,3]:=floattostr(EffPorAQ);
    MainForm.Soil2Grid.Cells[3,0]:=floattostr(LeachEff1);
    MainForm.Soil2Grid.Cells[3,1]:=floattostr(LeachEff2);
    MainForm.Soil2Grid.Cells[3,2]:=floattostr(LeachEffTR);
    MainForm.Soil2Grid.Cells[3,3]:=floattostr(LeachEffAQ);
    MainForm.Soil2Grid.Cells[4,0]:=floattostr(InitSaltRoot1);
    MainForm.Soil2Grid.Cells[4,1]:=floattostr(InitSaltRoot2);
    MainForm.Soil2Grid.Cells[4,2]:=floattostr(InitSaltTrans);
    MainForm.Soil2Grid.Cells[4,3]:=floattostr(InitSaltAquif);
  end;
  if ThreeLayer then
  begin
    readln(DataFile,str18,Str18A,Str18B,Str18C);
    TotPor1:=stringtofloat(Str18A);
    EffPor1:=stringtofloat(Str18B);
    LeachEff1:=stringtofloat(Str18C);
    readln(DataFile,str18,Str18A,Str18B,Str18C);
    TotPor2:=stringtofloat(Str18A);
    EffPor2:=stringtofloat(Str18B);
    LeachEff2:=stringtofloat(Str18C);
    readln(DataFile,str18,Str18A,Str18B,Str18C);
    TotPor3:=stringtofloat(Str18A);
    EffPor3:=stringtofloat(Str18B);
    LeachEff3:=stringtofloat(Str18C);
    readln(DataFile,str18,Str18A,Str18B,Str18C);
    TotPorTR:=stringtofloat(Str18A);
    EffPorTR:=stringtofloat(Str18B);
    LeachEffTR:=stringtofloat(Str18C);
    readln(DataFile,str18,Str18A,Str18B,Str18C);
    TotPorAQ:=stringtofloat(Str18A);
    EffPorAQ:=stringtofloat(Str18B);
    LeachEffAQ:=stringtofloat(Str18C);
    MainForm.Soil3Grid.Cells[1,0]:=floattostr(TotPor1);
    MainForm.Soil3Grid.Cells[1,1]:=floattostr(TotPor2);
    MainForm.Soil3Grid.Cells[1,2]:=floattostr(TotPor3);
    MainForm.Soil3Grid.Cells[1,3]:=floattostr(TotPorTR);
    MainForm.Soil3Grid.Cells[1,4]:=floattostr(TotPorAQ);
    MainForm.Soil3Grid.Cells[2,0]:=floattostr(EffPor1);
    MainForm.Soil3Grid.Cells[2,1]:=floattostr(EffPor2);
    MainForm.Soil3Grid.Cells[2,2]:=floattostr(EffPor3);
    MainForm.Soil3Grid.Cells[2,3]:=floattostr(EffPorTR);
    MainForm.Soil3Grid.Cells[2,4]:=floattostr(EffPorAQ);
    MainForm.Soil3Grid.Cells[3,0]:=floattostr(LeachEff1);
    MainForm.Soil3Grid.Cells[3,1]:=floattostr(LeachEff2);
    MainForm.Soil3Grid.Cells[3,2]:=floattostr(LeachEff3);
    MainForm.Soil3Grid.Cells[3,3]:=floattostr(LeachEffTR);
    MainForm.Soil3Grid.Cells[3,4]:=floattostr(LeachEffAQ);
    MainForm.Soil3Grid.Cells[4,0]:=floattostr(InitSaltRoot1);
    MainForm.Soil3Grid.Cells[4,1]:=floattostr(InitSaltRoot2);
    MainForm.Soil3Grid.Cells[4,2]:=floattostr(InitSaltRoot3);
    MainForm.Soil3Grid.Cells[4,3]:=floattostr(InitSaltTrans);
    MainForm.Soil3Grid.Cells[4,4]:=floattostr(InitSaltAquif);
  end;

  for i:=1 to 3 do readln (DataFile,Str12);
  if TwoLayer or ThreeLayer then for i:=1 to 3 do
     readln(DataFile, Str12);

//Reading measured soil salinity
  setlength(SaltRoot1M,NrOfSteps+1);
  setlength(SaltRoot2M,NrOfSteps+1);
  setlength(SaltRoot3M,NrOfSteps+1);
  setlength(SaltTransM,NrOfSteps+1);
  setlength(SaltAquifM,NrOfSteps+1);
  MainForm.SaltGrid1.Cells[0,0]:='Time step';
  MainForm.SaltGrid2.Cells[0,0]:='Time step';
  MainForm.SaltGrid3.Cells[0,0]:='Time step';

  for i:=1 to NrOfSteps do
  begin
    MainForm.SaltGrid1.Cells[0,i]:=inttostr(i);
    MainForm.SaltGrid2.Cells[0,i]:=inttostr(i);
    MainForm.SaltGrid3.Cells[0,i]:=inttostr(i);

    if OneLayer then
    begin
      readln(DataFile,Str10,Str12,Str18,Str10A);
      while pos (' ',Str12)>0 do delete(Str12,Pos(' ',Str12),1);
      if (Str12='-') or (Str12='') then
      begin
        SaltRoot1M[i]:=-1;
        MainForm.SaltGrid1.Cells[1,i]:='';
      end else
      begin
        SaltRoot1M[i]:=stringtofloat(Str12);
        MainForm.SaltGrid1.Cells[1,i]:=Str12;
      end;
      while pos (' ',Str18)>0 do delete(Str18,Pos(' ',Str18),1);
      if (Str18='-') or (Str18='') then
      begin
        SaltTransM[i]:=-1;
        MainForm.SaltGrid1.Cells[2,i]:='';
      end else
      begin
        SaltTransM[i]:=stringtofloat(Str18);
        MainForm.SaltGrid1.Cells[2,i]:=Str18;
      end;
      while pos (' ',Str10A)>0 do delete(Str10A,Pos(' ',Str10A),1);
      if (Str10A='-') or (Str10A='') then
      begin
        SaltAquifM[i]:=-1;
        MainForm.SaltGrid1.Cells[3,i]:='';
      end else
      begin
        SaltAquifM[i]:=stringtofloat(Str10A);
        MainForm.SaltGrid1.Cells[3,i]:=Str10A;
      end;
      MainForm.SaltGrid1.RowCount:=NrOfSteps+1;
    end;

    if TwoLayer then
    begin
      readln(DataFile,Str10,Str8,Str9,Str18,Str10A);
      while pos (' ',Str8)>0 do delete(Str8,Pos(' ',Str8),1);
      if (Str8='-') or (Str8='') then
      begin
        SaltRoot1M[i]:=-1;
        MainForm.SaltGrid2.Cells[1,i]:='';
      end else
      begin
        SaltRoot1M[i]:=stringtofloat(Str8);
        MainForm.SaltGrid2.Cells[1,i]:=Str8;
      end;
      while pos (' ',Str9)>0 do delete(Str9,Pos(' ',Str9),1);
      if (Str9='-') or (Str9='') then
      begin
        SaltRoot2M[i]:=-1;
        MainForm.SaltGrid2.Cells[2,i]:='';
      end else
      begin
        SaltRoot2M[i]:=stringtofloat(Str9);
        MainForm.SaltGrid2.Cells[2,i]:=Str9;
      end;
      while pos (' ',Str18)>0 do delete(Str18,Pos(' ',Str18),1);
      if (Str18='-') or (Str18='') then
      begin
        SaltTransM[i]:=-1;
        MainForm.SaltGrid2.Cells[3,i]:='';
      end else
      begin
        SaltTransM[i]:=stringtofloat(Str18);
        MainForm.SaltGrid2.Cells[3,i]:=Str18;
      end;
      while pos (' ',Str10A)>0 do delete(Str10A,Pos(' ',Str10A),1);
      if (Str10A='-') or (Str10='') then
      begin
        SaltAquifM[i]:=-1;
        MainForm.SaltGrid2.Cells[4,i]:='';
      end else
      begin
        SaltAquifM[i]:=stringtofloat(Str10A);
        MainForm.SaltGrid2.Cells[4,i]:=Str10A;
      end;
      MainForm.SaltGrid2.RowCount:=NrOfSteps+1;
    end;

    if ThreeLayer then
    begin
      readln(DataFile,Str10,Str8,Str8A,Str8B,Str18,Str10A);
      while pos (' ',Str8)>0 do delete(Str8,Pos(' ',Str8),1);
      if (Str8='-') or (Str8='') then
      begin
        SaltRoot1M[i]:=-1;
        MainForm.SaltGrid3.Cells[1,i]:='';
      end else
      begin
        SaltRoot1M[i]:=stringtofloat(Str8);
        MainForm.SaltGrid3.Cells[1,i]:=Str8;
      end;
      while pos (' ',Str8A)>0 do delete(Str8A,Pos(' ',Str8A),1);
      if (Str8A='-') or (Str8A='') then
      begin
        SaltRoot2M[i]:=-1;
        MainForm.SaltGrid3.Cells[2,i]:='';
      end else
      begin
        SaltRoot2M[i]:=stringtofloat(Str8A);
        MainForm.SaltGrid3.Cells[2,i]:=Str8A;
      end;
      while pos (' ',Str8B)>0 do delete(Str8B,Pos(' ',Str8B),1);
      if (Str8B='-') or (Str8B='') then
      begin
        SaltRoot3M[i]:=-1;
        MainForm.SaltGrid3.Cells[3,i]:='';
      end else
      begin
        SaltRoot3M[i]:=stringtofloat(Str8B);
        MainForm.SaltGrid3.Cells[3,i]:=Str8B;
     end;
      while pos (' ',Str18)>0 do delete(Str18,Pos(' ',Str18),1);
      if (Str18='-') or (Str18='') then
      begin
        SaltTransM[i]:=-1;
        MainForm.SaltGrid3.Cells[4,i]:='';
      end else
      begin
        SaltTransM[i]:=stringtofloat(Str18);
        MainForm.SaltGrid3.Cells[4,i]:=Str18;
      end;
      while pos (' ',Str10A)>0 do delete(Str10A,Pos(' ',Str10A),1);
      if (Str10A='-') or (Str10A='') then
      begin
        SaltAquifM[i]:=-1;
        MainForm.SaltGrid3.Cells[5,i]:='';
      end else
      begin
        SaltAquifM[i]:=stringtofloat(Str10A);
        MainForm.SaltGrid3.Cells[5,i]:=Str10A;
      end;
      MainForm.SaltGrid3.RowCount:=NrOfSteps+1;
    end;

  end; {for i:=1 to NrOfSteps do}

  closefile(DataFile);

end; {procedure ReadInput}



procedure WaterBalance;
var i, j, k, kk : integer;
    DevCalc, DevObs, DrQH1, Bal, SumBal, ExtraDr, EndDiff : real;
    Astr, Bstr : string;
    CalcDev : array of real;

begin
//Initial settings
  if OneLayer then StorMax:=1000*(Layer1*(TotPor1-EffPor1));
  if TwoLayer then StorMax:=1000*(Layer1*(TotPor1-Effpor1)
                            +Layer2*(TotPor2-EffPor2));
  if ThreeLayer then StorMax:=1000*(Layer1*(TotPor1-EffPor1)
                            +Layer2*(TotPor2-EffPor2)+Layer3*(TotPor3-EffPor3));
  Stor[0]:=0;
  StorTot[0]:=Moisture*StorMax/100;
  setlength (CalcDev,NrOfSteps+1);
  FinalDiff:=100000;
  EndDiff:=100000;
  DrQH1:=DrainQH1;
  DrainQH1:=0;

//Trials to find optimal drainage capacity DrainQH1
  FOR KK:=1 to 200 DO
  BEGIN
    DepthWT[0]:=InitDepthWT;
    DrainQH1:=DrainQH1+0.001;
    SumDiffSq:=0;
    SumBal:=0;
  for i:=1 to NrOfSteps do              {Start calculations for all times teps}
  begin
//Hydrological factors per day, initial settings for summations
    DRCT:=0;
    EPAT:=0;
    PERCT:=0;
    CAPT:=0;
//Determinimg surface drainage and adjusting irrigation
    for j:=1 to StepLength do              {Calculations whithin the time step}
    begin
//Choosing factor ZZ in dependance of DepthWT and CDW for determining
//capillary rise and adjustment of surface runoff
      CDWR:=CDW-0.5*RootZone;                             {depth rootzone in m}
      IF CDWR<=0 THEN
        ZZ:=1
      ELSE
      BEGIN
        ZZ:=(CDW-DepthWT[i-1])/CDWR;
        IF ZZ<0 THEN ZZ:=0;
        IF ZZ>1 THEN ZZ:=1;
      END;
//Determining capillary rise (CAP) and actual evaporation EPA
      ES:=Rain[i]+Irrig[i]; {Surface water resources}
      if IrrEff>0 then
      begin
        if ES>=Evap[i]/IrrEff then
        begin
          EPA:=Evap[i];
          Cap:=0;
        end else
        begin
          EPA:=IrrEff*ES+ZZ*(Evap[i]-IrrEff*ES);       {actual evaporation}
          CAP:=EPA-ES;                                      {Capillary rise}
          IF CAP<0 then CAP:=0;
        end;
      end;
      if EPA>EVAP[i] then EPA:=EVAP[i];

      if StepLength=1 then
      begin
        Stor[i]:=Rain[i]+Irrig[i]-EPA;                                {storage}
        if Stor[i]+Stor[i-1]>StorMax then
           Stor[i]:=StorMax-Stor[i-1];
        StorTot[i]:=StorTot[i-1]+Stor[i];
        if StorTot[i]<0 then StorTot[i]:=0;
        if Stor[i]<0 then Stor[i]:=0;
        if StorTot[i]>StorMax then
        begin
          StorTot[i]:=StorMax;
          Stor[i]:=0;
        end;
        Perc:=Rain[i]+Irrig[i]-EPA-Stor[i];
        if Perc<0 then Perc:=0;
        if Perc>Cap then                                           {net values}
        begin
          Perc:=Perc-Cap;
          Cap:=0;
        end else
        begin
          Cap:=Cap-Perc;
          Perc:=0;
        end;
      end else
        Perc:=Rain[i]+Irrig[i]-EPA;

//Adjusting percolation and capillary rise for rootzone layers;
      EPA1:=0;
      EPA2:=0;
      EPA3:=0;
      if OneLayer then
      begin
        Perc1[i,j]:=Perc;
        EPA1:=EPA;
        Cap1[i,j]:=Cap;
      end;
      if TwoLayer then
      begin
        EPA1:=(2/3+Ratio1)*EPA;
        EPA2:=(1/3+Ratio2)*EPA;
        Perc1[i,j]:=(Rain[i]+Irrig[i])/IrrEff-EPA1; {Percolation from layer 1}
        Perc2[i,j]:=Perc1[i,j]-EPA2;                {Percolation from layer 2}
        if Perc2[i,j]<0 then Perc2[i,j]:=0;
        Cap3[i,j]:=Cap;
        Cap2[i,j]:=Cap3[i,j]-EPA2;         {Cap. rise from 2nd layer into 1st}
        if Cap2[i,j]<0 then Cap2[i,j]:=0;
      end;
      if ThreeLayer then
      begin
        EPA1:=(0.5+Ratio1)*EPA;
        EPA2:=(1/3+Ratio2)*EPA;
        EPA3:=(1/6+Ratio3)*EPA;
        Perc1[i,j]:=(Rain[i]+Irrig[i])/IrrEff-EPA1; {Percolation from layer 1}
        if Perc1[i,j]<0 then Perc1[i,j]:=0;
        Perc2[i,j]:=Perc1[i,j]-EPA2;                {Percolation from layer 2}
        if Perc2[i,j]<0 then Perc2[i,j]:=0;
        Perc3[i,j]:=Perc2[i,j]-EPA3;                {Percolation from layer 3}
        if Perc3[i,j]<0 then Perc3[i,j]:=0;
        Cap4[i,j]:=Cap;
        Cap3[i,j]:=Cap4[i,j]-EPA3;                  {Cap. rise into 2nd layer}
        if Cap3[i,j]<0 then Cap3[i,j]:=0;
        Cap2[i,j]:=Cap3[i,j]-EPA2;         {Cap. rise from 2nd layer into 1st}
        if Cap2[i,j]<0 then Cap2[i,j]:=0;
      end;

//Determining the subsurface drainage
     if DrainQH1<0.02 then
     begin
        HCR:=DrainDepth-DepthWT[i-1];                                            {in m}
        if (HCR<=0) or not DrainSystem  then
        begin
          DRA:=0;
          DRB:=0;
          DRC:=0;
        end;
        if (HCR>0) and DrainSystem then
        begin
          DRA:=1000*DrainQH2*HCR*HCR;                         {from m to mm}
          DRB:=1000*DrainQH1*HCR;
          IF (DepthWT[i-1]<=-0.1) then DRA:=DRA*2;
          DRC:=DRA+DRB;
          if DRC<0 then DRC:=0;
        end;
//Determining effective/drainable porosity
        if (DepthWT[i-1]<=0) then EffPorosity:=1;
        if OneLayer then
           if (DepthWT[i-1]>0) and (DepthWT[i-1]<=RootZone) then
               Effporosity:=EffPor1;
        if TwoLayer then
        begin
          if (DepthWT[i-1]>0) and (DepthWT[i-1]<=Layer1) then
              Effporosity:=EffPor1;
          if (DepthWT[i-1]>Layer1) and (DepthWT[i-1]<=Layer2) then
              Effporosity:=EffPor2;
        end;
        if ThreeLayer then
        begin
          if (DepthWT[i-1]>0) and (DepthWT[i-1]<=Layer1) then
              Effporosity:=EffPor1;
          if (DepthWT[i-1]>Layer1) and (DepthWT[i-1]<=Layer2) then
              Effporosity:=EffPor2;
          if (DepthWT[i-1]>Layer2) and (DepthWT[i-1]<=Layer3) then
              Effporosity:=EffPor3;
        end;
        if (DepthWT[i-1]>RootZone) and (DepthWT[i-1]<=RootZone+Trans) then
            Effporosity:=EffPorTR;
        if (DepthWT[i-1]>RootZone+Trans) then Effporosity:=EffPorAQ;
//New depth water table
        DepthWT[i]:=DepthWT[i-1]
                    -(PERC-CAP-DRC+GrwtInf-GrwtOut)/(1000*Effporosity);
     end; {if DrainQH1<0.02}

     if DrainQH1>=0.02 then
     begin
//Calculating subs. drainage with 10 steps, converting data
       Perc:=Perc/10;
       Cap:=Cap/10;
       GrwtInf:=GrwtInf/10;
       GrwtOut:=GrwtOut/10;
       DRCk:=0;
       DWTi:=DepthWT[i-1];
       for k:=1 to 10 do
       begin
           HCR:=DrainDepth-DWTi;                                            {in m}
           IF (HCR<=0) or not DrainSystem  then
           BEGIN
             DRA:=0;
             DRB:=0;
             DRC:=0;
           END ELSE
           BEGIN
             DRA:=1000*DrainQH2*HCR*HCR/10;                      {from m to mm}
             DRB:=1000*DrainQH1*HCR/10;
             IF (DepthWT[k]<=-0.1) THEN DRA:=DRA*2;
             DRC:=DRA+DRB;
             if DRC<0 then DRC:=0;
           END;
           DRCk:=DRCk+DRC;
//Determining effective/drainable porosity
           if (DWTi<=0) then EffPorosity:=1;
           if OneLayer then
              if (DWTi>0) and (DWTi<=RootZone) then Effporosity:=EffPor1;
           if TwoLayer then
           begin
             if (DWTi>0) and (DWTi<=Layer1) then Effporosity:=EffPor1;
             if (DWTi>Layer1) and (DWTi<=Layer2) then Effporosity:=EffPor2;
           end;
           if ThreeLayer then
           begin
             if (DWTi>0) and (DWTi<=Layer1) then Effporosity:=EffPor1;
             if (DWTi>Layer1) and (DWTi<=Layer2) then Effporosity:=EffPor2;
             if (DWTi>Layer2) and (DWTi<=Layer3) then Effporosity:=EffPor3;
           end;
            if (DWTi>RootZone) and (DWTi<=RootZone+Trans) then
               Effporosity:=EffPorTR;
           if (DWTi>RootZone+Trans) then Effporosity:=EffPorAQ;
//New depth water table
           DWTi:=DWTi-(PERC-CAP-DRC+GrwtInf-GrwtOut)/(1000*Effporosity);
       end; {for k:=1 to 10 do}
       DRC:=DRCk;
       DepthWT[i]:=DWTi;
//Reconverting data after ending 10 smaller timesteps;
       Perc:=Perc*10;
       Cap:=Cap*10;
       GrwtInf:=GrwtInf*10;
       GrwtOut:=GrwtOut*10;
     end; {if DrainQH1>=0.02 then}

//Summations of drainage, evaporation and capillary per day
      DRCT:=DRCT+DRC;
      EPAT:=EPAT+EPA;
      PERCT:=PERCT+PERC;
      CAPT:=CAPT+CAP;
    end; {for j:=1 to StepLength do}

//Waterbalance
    Bal:=abs(PERCT-CAPT-DRCT+(GrwtInf-GrwtOut)*StepLength);
    SumBal:=SumBal+Bal;


//Optimization
   if abs(SumBal)<FinalDiff then
   begin
     FinalDiff:=abs(SumBal);
     DrainQH1F:=DrainQH1;
   end;

   end; {for i:=1 to NrOfSteps do}

   Impr:=true;
   if (DrainQH1F>0.199) or (DrainQH1F<0.002) then
   begin
     DrainQH1F:=DrQH1;
     Impr:=false;
   end;

  END; {for kk:=1 to 200 do}

end; {procedure WaterBalance}


procedure FinalBalance;
var i, j, k : integer; Bal, SumBal, SumIrr, SumIrrig, ExtraDr, AvDiff : real;

begin
//Initial settings
  SumBal:=0;
  SumIrr:=0;
  SumIrrig:=0;
  DEPTHWT[0]:=InitDepthWT;
  if OneLayer then StorMax:=1000*(Layer1*(TotPor1-EffPor1));
  if TwoLayer then StorMax:=1000*(Layer1*(TotPor1-Effpor1)
                            +Layer2*(TotPor2-EffPor2));
  if ThreeLayer then StorMax:=1000*(Layer1*(TotPor1-EffPor1)
                            +Layer2*(TotPor2-EffPor2)+Layer3*(TotPor3-EffPor3));
  StorTot[0]:=Moisture*StorMax/100;
  Stor[0]:=0;

  for i:=1 to NrOfSteps do              {Start calculations for all times teps}
  begin
//Hydrological factors per day, initial settings for summations
    DRCT:=0;
    EPAT:=0;
    PERCT:=0;
    CAPT:=0;

    for j:=1 to StepLength do              {Calculations whithin the time step}
    begin
//Choosing factor ZZ in dependance of DepthWT and CDW for determining
//capillary rise and adjustment of surface runoff
      CDWR:=CDW-0.5*RootZone;                             {depth rootzone in m}
      IF CDWR<=0 THEN
        ZZ:=1
      ELSE
      BEGIN
        ZZ:=(CDW-DepthWT[i-1])/CDWR;
        IF ZZ<0 THEN ZZ:=0;
        IF ZZ>1 THEN ZZ:=1;
      END;
//Determining capillary rise (CAP) and actual evaporation EPA
      ES:=Rain[i]+Irrig[i]; {Surface water resources}
      if IrrEff>0 then
      begin
        if ES>=Evap[i]/IrrEff then
        begin
          EPA:=Evap[i];
          Cap:=0;
        end else
        begin
          EPA:=IrrEff*ES+ZZ*(Evap[i]-IrrEff*ES);       {actual evaporation}
          CAP:=EPA-ES;                                      {Capillary rise}
          IF CAP<0 then CAP:=0;
        end;
      end;
      if EPA>EVAP[i] then EPA:=EVAP[i];

      if StepLength=1 then
      begin
        Stor[i]:=Rain[i]+Irrig[i]-EPA;                                {storage}
        if Stor[i]+Stor[i-1]>StorMax then
           Stor[i]:=StorMax-Stor[i-1];
        StorTot[i]:=StorTot[i-1]+Stor[i];
        if StorTot[i]<0 then StorTot[i]:=0;
        if Stor[i]<0 then Stor[i]:=0;
        if StorTot[i]>StorMax then
        begin
          StorTot[i]:=StorMax;
          Stor[i]:=0;
        end;
        Perc:=Rain[i]+Irrig[i]-EPA-Stor[i];
        if Perc<0 then Perc:=0;
        if Perc>Cap then                                           {net values}
        begin
          Perc:=Perc-Cap;
          Cap:=0;
        end else
        begin
          Cap:=Cap-Perc;
          Perc:=0;
        end;
      end else
        Perc:=Rain[i]+Irrig[i]-EPA;

//Adjusting percolation and capillary rise for rootzone layers;
      EPA1:=0;
      EPA2:=0;
      EPA3:=0;
      if OneLayer then
      begin
        Perc1[i,j]:=Perc;
        EPA1:=EPA;
        Cap1[i,j]:=Cap;
      end;
      if TwoLayer then
      begin
        EPA1:=(2/3+Ratio1)*EPA;
        EPA2:=(1/3+Ratio2)*EPA;
        Perc1[i,j]:=(Rain[i]+Irrig[i])/IrrEff-EPA1; {Percolation from layer 1}
        Perc2[i,j]:=Perc1[i,j]-EPA2;                {Percolation from layer 2}
        if Perc2[i,j]<0 then Perc2[i,j]:=0;
        Cap3[i,j]:=Cap;
        Cap2[i,j]:=Cap3[i,j]-EPA2;         {Cap. rise from 2nd layer into 1st}
        if Cap2[i,j]<0 then Cap2[i,j]:=0;
      end;
      if ThreeLayer then
      begin
        EPA1:=(0.5+Ratio1)*EPA;
        EPA2:=(1/3+Ratio2)*EPA;
        EPA3:=(1/6+Ratio3)*EPA;
        Perc1[i,j]:=(Rain[i]+Irrig[i])/IrrEff-EPA1; {Percolation from layer 1}
        if Perc1[i,j]<0 then Perc1[i,j]:=0;
        Perc2[i,j]:=Perc1[i,j]-EPA2;                {Percolation from layer 2}
        if Perc2[i,j]<0 then Perc2[i,j]:=0;
        Perc3[i,j]:=Perc2[i,j]-EPA3;                {Percolation from layer 3}
        if Perc3[i,j]<0 then Perc3[i,j]:=0;
        Cap4[i,j]:=Cap;
        Cap3[i,j]:=Cap4[i,j]-EPA3;                  {Cap. rise into 2nd layer}
        if Cap3[i,j]<0 then Cap3[i,j]:=0;
        Cap2[i,j]:=Cap3[i,j]-EPA2;         {Cap. rise from 2nd layer into 1st}
        if Cap2[i,j]<0 then Cap2[i,j]:=0;
      end;

//Determining the subsurface drainage
     if DrainQH1F<0.02 then
     begin
        HCR:=DrainDepth-DepthWT[i-1];                                            {in m}
        if (HCR<=0) or not DrainSystem  then
        begin
          DRA:=0;
          DRB:=0;
          DRC:=0;
        end;
        if (HCR>0) and DrainSystem then
        begin
          DRA:=1000*DrainQH2*HCR*HCR;                         {from m to mm}
          DRB:=1000*DrainQH1F*HCR;
          IF (DepthWT[i-1]<=-0.1) then DRA:=DRA*2;
          DRC:=DRA+DRB;
          if DRC<0 then DRC:=0;
        end;
//Determining effective/drainable porosity
        if (DepthWT[i-1]<=0) then EffPorosity:=1;
        if OneLayer then
           if (DepthWT[i-1]>0) and (DepthWT[i-1]<=RootZone) then
               Effporosity:=EffPor1;
        if TwoLayer then
        begin
          if (DepthWT[i-1]>0) and (DepthWT[i-1]<=Layer1) then
              Effporosity:=EffPor1;
          if (DepthWT[i-1]>Layer1) and (DepthWT[i-1]<=RootZone) then
              Effporosity:=EffPor2;
        end;
        if ThreeLayer then
        begin
          if (DepthWT[i-1]>0) and (DepthWT[i-1]<=Layer1) then
              Effporosity:=EffPor1;
          if (DepthWT[i-1]>Layer1) and (DepthWT[i-1]<=Layer1+Layer2) then
              Effporosity:=EffPor2;
          if (DepthWT[i-1]>Layer2) and (DepthWT[i-1]<=RootZone) then
              Effporosity:=EffPor3;
        end;
        if (DepthWT[i-1]>RootZone) and (DepthWT[i-1]<=RootZone+Trans) then
            Effporosity:=EffPorTR;
        if (DepthWT[i-1]>RootZone+Trans) then Effporosity:=EffPorAQ;
//New depth water table
        DepthWT[i]:=DepthWT[i-1]
                    -(PERC-CAP-DRC+GrwtInf-GrwtOut)/(1000*Effporosity);
     end; {if DrainQH1F<0.02}

     if DrainQH1F>=0.02 then
     begin
//Calculating subs. drainage with 10 steps, converting data
       Perc:=Perc/10;
       Cap:=Cap/10;
       GrwtInf:=GrwtInf/10;
       GrwtOut:=GrwtOut/10;
       DRCk:=0;
       DWTi:=DepthWT[i-1];
       for k:=1 to 10 do
       begin
           HCR:=DrainDepth-DWTi;                                            {in m}
           IF (HCR<=0) or not DrainSystem  then
           BEGIN
             DRA:=0;
             DRB:=0;
             DRC:=0;
           END ELSE
           BEGIN
             DRA:=1000*DrainQH2*HCR*HCR/10;                      {from m to mm}
             DRB:=1000*DrainQH1F*HCR/10;
             IF (DepthWT[k]<=-0.1) THEN DRA:=DRA*2;
             DRC:=DRA+DRB;
             if DRC<0 then DRC:=0;
           END;
           DRCk:=DRCk+DRC;
//Determining effective/drainable porosity
           if (DWTi<=0) then EffPorosity:=1;
           if OneLayer then
              if (DWTi>0) and (DWTi<=RootZone) then Effporosity:=EffPor1;
           if TwoLayer then
           begin
             if (DWTi>0) and (DWTi<=Layer1) then Effporosity:=EffPor1;
             if (DWTi>Layer1) and (DWTi<=RootZone) then Effporosity:=EffPor2;
           end;
           if ThreeLayer then
           begin
             if (DWTi>0) and (DWTi<=Layer1) then Effporosity:=EffPor1;
             if (DWTi>Layer1) and (DWTi<=Layer1+Layer2) then
                 Effporosity:=EffPor2;
             if (DWTi>Layer2) and (DWTi<=RootZone) then Effporosity:=EffPor3;
           end;
            if (DWTi>RootZone) and (DWTi<=RootZone+Trans) then
               Effporosity:=EffPorTR;
           if (DWTi>RootZone+Trans) then Effporosity:=EffPorAQ;
//New depth water table
           DWTi:=DWTi-(PERC-CAP-DRC+GrwtInf-GrwtOut)/(1000*Effporosity);
       end; {for k:=1 to 10 do}
       DRC:=DRCk;
       DepthWT[i]:=DWTi;
//Reconverting data after ending 10 smaller timesteps;
       Perc:=Perc*10;
       Cap:=Cap*10;
       GrwtInf:=GrwtInf*10;
       GrwtOut:=GrwtOut*10;
     end; {if DrainQH1F>=0.02 then}

     if DepthWT[i]>RootZone+Trans+Aquif then
     begin
       showmessage ('The water table is below the bottom of the aquifer.'+
                    ' Calculations stopped.  Please adjust data.');
       TooDeep:=true;
       exit;
     end;

//Summations of drainage, evaporation and capillary per day
      DRCT:=DRCT+DRC;
      EPAT:=EPAT+EPA;
      PERCT:=PERCT+PERC;
      CAPT:=CAPT+CAP;
    end; {for j:=1 to StepLength do}

//For saltbalance and output
    PercF[i]:=PercT;
    CapF[i]:=CapT;
    EPAf[i]:=EPAT;
    DRCf[i]:=DRCT;
    Bal:=abs(PERC-CAP-DRC+GrwtInf-GrwtOut);
    SumBal:=SumBal+Bal;

//checking irrigation
    SumIrr:=SumIrr+Irri[i];
    SumIrrig:=SumIrrig+Irrig[i];

   end; {for i:=1 to NrOfSteps do}

end; {procedure FinalBalance}


procedure OneLayerRootzone;
var i, j, k : integer;
begin
//Optimizing leaching efficiency Layer 1 in case of one soil layer
  LeachEff := 0.1;
  FinalDiff := 1000000;
  for j:=1 to 180 do
  begin
    LeachEff := LeachEff+0.01;
    SumDiffSq := 0;
    Salinity:=SaltLayer1i;
//The two below can be made into one
    SaltCap:=InitSaltTrans;
    SaltTrans:=InitSaltTrans;
    SaltAquif:=InitSaltAquif;
    Salinity:=SaltLayer1i;

    for i:=1 to NrOfSteps do for k:=1 to Steplength do
    begin
//Salinity rootzone
      InitSaltR:=Salinity;
      SaltIn:=(Irrig[i]*SaltIrrig+Cap1[i,k]*SaltCap)/WaterDepth1;  
      SaltOut:=0.5*LeachEff*Perc1[i,k]*Salinity/WaterDepth1;
      Divider:=1+0.5*LeachEff*Perc1[i,k]/WaterDepth1;
      if Divider<0.001 then Divider:=0.001;
      Salinity:=(Salinity+SaltIn-SaltOut)/Divider;
      if Salinity<=0 then Salinity:=0.1;
      if (SaltRoot1M[i]>0) and (k=Steplength) then
      begin
        Diff := sqr(Salinity-SaltRoot1M[i]);
        SumDiffSq:=SumDiffSq+Diff;
        if SumDiffSq>1000 then SumDiffSq:=1000;
      end;
      AveSalt:=(InitSaltR+Salinity)/2;
//Salinity transition zone
      InitSaltT:=SaltTrans;
      SaltIn:=(PERC*AveSalt*LeachEff+GrwtUp*SaltAquif)/WaterDepthTR;
      SaltOut:= 0.5*(DRCf[i]*LeachEffTR*SaltTrans+GrwtDown*LeachEffTR*SaltTrans)
                /WaterDepthTR;
      Divider:=1+0.5*(DRCf[i]*LeachEffTR+GrwtDown*LeachEffTR)
               /WaterDepthTR;
      if Divider<0.001 then Divider:=0.001;
      SaltTrans:=(SaltTrans+SaltIn-SaltOut)/Divider;
      if SaltTrans<0 then SaltTrans:=0.1;
      SaltCap:=SaltTrans;
      AveSalt:=(InitSaltT+SaltTrans)/2;
//Salinity aquifer
      SaltIn:=(GrwtDown*AveSalt*LeachEffTR+GrwtInf*SalInf*StepLength)/WaterDepthAQ;
      SaltOut:=0.5*(GrwtOut*StepLength+GrwtUp)*LeachEffAQ*SaltAquif/WaterDepthAQ;
      Divider:= 1+0.5*(GrwtOut*StepLength+GrwtUp)*LeachEffAQ/WaterDepthAQ;
      if Divider<0.001 then Divider:=0.001;
      SaltAquif:=(SaltAquif+SaltIn-SaltOut)/Divider;
      if SaltAquif<0 then SaltAquif:=0.1;
    end; {for i:=1 to NrOfSteps do for k:=1 to Steplength do}
//Determining final optimimum of leaching efficiency root zone with 1 layer
    if (SumDiffSq<FinalDiff) then
    begin
      FinalDiff:=SumDiffSq;
      LeachEff1:=LeachEff;
    end;
  end;{for j:=1 to 100 do}

//Finding the coefficient of explanation Expl1 measured and calculated salinity
  if Count1>0 then
  begin
    SumSalt:=0;
    for i:=1 to NrOfSteps do if SaltRoot1M[i]>0 then
        SumSalt:=SumSalt+SaltRoot1M[i];
    Ave:=SumSalt/Count1;
    SumAvSq:=0;
    for i:=1 to NrOfSteps do
    begin
      if SaltRoot1M[i]>0 then
         SumAvSq:=SumAvSq+Sqr(SaltRoot1M[i]);
    end;
    Expl1:=1-sqrt(FinalDiff/SumAvSq);
  end;
end; {procedure OneLayerRootzone}



procedure OneLayerTranszone;
var i, j, k : integer;
begin
//Optimizing leaching efficiency Layer 1 in case of one soil layer
  LeachEff := 0.1;
  FinalDiff := 1000000;
  for j:=1 to 180 do
  begin
    LeachEff := LeachEff+0.01;
    SumDiffSq := 0;
    Salinity:=InitSaltTrans;
//The two below can be made into one
    SaltCap:=InitSaltTrans;
    SaltTrans:=InitSaltTrans;
    Salinity:=InitSaltTrans;
    SaltAquif:=InitSaltAquif;
    SaltLayer1:=Saltlayer1i;

    for i:=1 to NrOfSteps do for k:=1 to StepLength do
    begin
//Salinity rootzone
      InitSaltR:=SaltLayer1;
      SaltIn:=(Irrig[i]*SaltIrrig+Cap1[i,k]*SaltCap)/WaterDepth1;
      SaltOut:=0.5*LeachEff1*Perc1[i,k]*SaltLayer1/WaterDepth1;
      Divider:=1+0.5*LeachEff1*Perc1[i,k]/WaterDepth1;
      if Divider<0.001 then Divider:=0.001;
      SaltLayer1:=(SaltLayer1+SaltIn-SaltOut)/Divider;
      AveSalt:=(InitSaltR+SaltLayer1)/2;
//Salinity transition zone
      InitSaltT:=Salinity;
      SaltIn:=(Perc1[i,k]*AveSalt*LeachEff1+GrwtUp*SaltAquif)/WaterDepthTR;
      SaltOut:= 0.5*(DRCf[i]*LeachEff*Salinity+GrwtDown*LeachEff*SaltTrans)
                /WaterDepthTR;
      Divider:=1+0.5*(DRCf[i]*LeachEff+GrwtDown*LeachEff)/WaterDepthTR;
      if Divider<0.001 then Divider:=0.001;
      Salinity:=(Salinity+SaltIn-SaltOut)/Divider;
      if Salinity<0 then Salinity:=0.1;
      SaltCap:=Salinity;
      AveSalt:=(InitSaltT+Salinity)/2;
      if (SaltTransM[i]>0) and (k=Steplength) then
      begin
        Diff := sqr(Salinity-SaltTRansM[i]);
        SumDiffSq:=SumDiffSq+Diff;
        if SumDiffSq>1000 then SumDiffSq:=1000;
      end;
//Salinity aquifer
      SaltIn:=(GrwtDown*AveSalt*LeachEff+GrwtInf*SalInf*StepLength)/WaterDepthAQ;
      SaltOut:=(0.5*(GrwtOut*StepLength+GrwtUp)*LeachEffAQ*SaltAquif)/WaterDepthAQ;
      Divider:= 1+0.5*(GrwtOut*StepLength+GrwtUp)*LeachEffAQ/WaterDepthAQ;
      if Divider<0.001 then Divider:=0.001;
      SaltAquif:=(SaltAquif+SaltIn-SaltOut)/Divider;
      if SaltAquif<0 then SaltAquif:=0.1;
    end; {for i:=1 to NrOfSteps do for k:=1 to Steplength do}
//Determining final optimimum of leaching efficiency trans. zone with 1 layer
    if (SumDiffSq<FinalDiff) then
    begin
      FinalDiff:=SumDiffSq;
      LeachEffTR:=LeachEff;
    end;
  end;{for j:=1 to 100 do}

//Finding the coeff. of expl. ExplTR for measured and calculated salinity
  if Count4>0 then
  begin
    SumSalt:=0;
    for i:=1 to NrOfSteps do if SaltTransM[i]>0 then
        SumSalt:=SumSalt+SaltTransM[i];
    Ave:=SumSalt/Count4;
    SumAvSq:=0;
    for i:=1 to NrOfSteps do
    begin
      if SaltTransM[i]>0 then
         SumAvSq:=SumAvSq+Sqr(SaltTransM[i]);
    end;
    ExplTR:=1-sqrt(FinalDiff/SumAvSq);
  end;

end; {procedure OneLayerTranszone;}



procedure OneLayerAquifer;
var i, j, k : integer;
begin
//Optimizing leaching efficiency Layer 1 in case of one soil layer
  LeachEff := 0.1;
  FinalDiff := 1000000;
  for j:=1 to 180 do
  begin
    SumDiffSq := 0;
    LeachEff := LeachEff+0.01;
//The two below can be made into one
    SaltCap:=InitSaltTrans;
    SaltTrans:=InitSaltTrans;
    SaltAquif:=InitSaltAquif;
    Salinity:=InitSaltAquif;
    SaltLayer1:=Saltlayer1i;

    for i:=1 to NrOfSteps do for k:=1 to StepLength do
    begin
//Salinity rootzone
      InitSaltR:=SaltLayer1;
      SaltIn:=(Irrig[i]*SaltIrrig+Cap1[i,k]*SaltCap)/WaterDepth1;
      SaltOut:=0.5*LeachEff1*Perc1[i,k]*SaltLayer1/WaterDepth1;
      Divider:=1+0.5*LeachEff1*Perc1[i,k]/WaterDepth1;
      if Divider<0.001 then Divider:=0.001;
      SaltLayer1:=(SaltLayer1+SaltIn-SaltOut)/Divider;
      AveSalt:=(InitSaltR+SaltLayer1)/2;
//Salinity transition zone
      InitSaltT:=SaltTrans;
      SaltIn:=(Perc1[i,k]*AveSalt*LeachEff1+GrwtUp*SaltAquif)/WaterDepthTR;
      SaltOut:= 0.5*(DRCf[i]*LeachEffTR*SaltTrans+GrwtDown*LeachEffTR*SaltTrans)
                /WaterDepthTR;
      Divider:=1+0.5*(DRCf[i]*LeachEffTR+GrwtDown*LeachEffTR)
               /WaterDepthTR;
      if Divider<0.001 then Divider:=0.001;
      SaltTrans:=(SaltTrans+SaltIn-SaltOut)/Divider;
      if SaltTrans<0 then SaltTrans:=0.1;
      SaltCap:=SaltTrans;
      AveSalt:=(InitSaltT+SaltTrans)/2;
//Salinity aquifer
      SaltIn:=(GrwtDown*AveSalt*LeachEffTR+GrwtInf*SalInf*StepLength)/WaterDepthAQ;
      SaltOut:=(0.5*(GrwtOut*StepLength+GrwtUp)*LeachEff*Salinity)/WaterDepthAQ;
      Divider:= 1+0.5*(GrwtOut*StepLength+GrwtUp)*LeachEffAQ/WaterDepthAQ;
      if Divider<0.001 then Divider:=0.001;
      Salinity:=(Salinity+SaltIn-SaltOut)/Divider;
      if Salinity<0 then Salinity:=0.1;
      if (SaltAquifM[i]>0) and (k=Steplength) then
      begin
        Diff := sqr(Salinity-SaltAquifM[i]);
        SumDiffSq:=SumDiffSq+Diff;
        if SumDiffSq>1000 then SumDiffSq:=1000;
      end;
    end; {for i:=1 to NrOfSteps do for k:=1 to Steplength do}
//Determining final optimimum of leaching efficiency aquifer with 1 layer
    if (SumDiffSq<FinalDiff) then
    begin
      FinalDiff:=SumDiffSq;
      LeachEffAQ:=LeachEff;
    end;
  end;{for j:=1 to 100 do}
//Finding the coeff. of expl. ExplTR for measured and calculated salinity
  if Count5>0 then
  begin
    SumSalt:=0;
    for i:=1 to NrOfSteps do if SaltAquifM[i]>0 then
        SumSalt:=SumSalt+SaltAquifM[i];
    Ave:=SumSalt/Count5;
    SumAvSq:=0;
    for i:=1 to NrOfSteps do
    begin
      if SaltAquifM[i]>0 then
         SumAvSq:=SumAvSq+Sqr(SaltAquifM[i]);
    end;
    ExplAQ:=1-sqrt(FinalDiff/SumAvSq);
  end;

end; {procedure OneLayerAquif}



procedure TwoLayer1stPart;
var i, j, k : integer;
begin
//Optimizing leaching efficiency Layer 1
  LeachEff := 0.1;
  FinalDiff := 1000000;
  for j:=1 to 180 do
  begin
    LeachEff := LeachEff+0.01;
    SumDiffSq := 0;
    Salinity:=SaltLayer1i;
//The two below can be made into one
    SaltCap:=SaltLayer2i;
    SaltCap2:=InitSaltTrans;
    SaltTrans:=InitSaltTrans;
    SaltAquif:=InitSaltAquif;
    SaltLayer2:=SaltLayer2i;

    for i:=1 to NrOfSteps do for k:=1 to StepLength do
    begin
//Salinity 1st Layer
      InitSaltR:=Salinity;
      SaltIn:=(Irrig[i]*SaltIrrig+Cap2[i,k]*SaltCap)/WaterDepth1;
      SaltOut:=0.5*LeachEff*Perc1[i,k]*Salinity/WaterDepth1;
      Divider:=1+0.5*LeachEff*Perc1[i,k]/WaterDepth1;
      if Divider<0.001 then Divider:=0.001;
      Salinity:=(Salinity+SaltIn-SaltOut)/Divider;
      if Salinity<=0 then Salinity:=0.1;
      if (SaltRoot1M[i]>0) and (k=Steplength) then
      begin
        Diff := sqr(Salinity-SaltRoot1M[i]);
        SumDiffSq:=SumDiffSq+Diff;
        if SumDiffSq>1000 then SumDiffSq:=1000;
      end;
      AveSalt:=(InitSaltR+Salinity)/2;
//Salinity 2nd Layer
      InitSalt2:=SaltLayer2;
      SaltIn:=(Perc1[i,k]*AveSalt*LeachEff1+Cap3[i,k]*SaltCap2)/WaterDepth2;
      SaltOut:=0.5*LeachEff*(Perc2[i,k]+Cap2[i,k])*SaltLayer2/WaterDepth2;
      Divider:=1+0.5*LeachEff*(Perc2[i,k]+Cap2[i,k])/WaterDepth2;
      if Divider<0.001 then Divider:=0.001;
      SaltLayer2:=(SaltLayer2+SaltIn-SaltOut)/Divider;
      if SaltLayer2<=0 then SaltLayer2:=0.1;
      SaltCap:=SaltLayer2;
      AveSalt:=(InitSalt2+SaltLayer2)/2;
//Salinity transition zone
      InitSaltT:=SaltTrans;
      SaltIn:=(Perc2[i,k]*AveSalt*LeachEff+GrwtUp*SaltAquif)/WaterDepthTR;
      SaltOut:= 0.5*(DRCf[i]*LeachEffTR*SaltTrans+GrwtDown*LeachEffTR*SaltTrans)
                /WaterDepthTR;
      Divider:=1+0.5*(DRCf[i]*LeachEffTR+GrwtDown*LeachEffTR)
               /WaterDepthTR;
      if Divider<0.001 then Divider:=0.001;
      SaltTrans:=(SaltTrans+SaltIn-SaltOut)/Divider;
      if SaltTrans<0 then SaltTrans:=0.1;
      SaltCap2:=SaltTrans;
      AveSalt:=(InitSaltT+SaltTrans)/2;
//Salinity aquifer
      SaltIn:=(GrwtDown*AveSalt+GrwtInf*SalInf)/WaterDepthAQ;
      SaltOut:=(0.5*(GrwtOut*+GrwtUp)*LeachEffAQ*SaltAquif)/WaterDepthAQ;
      Divider:= 1+0.5*(GrwtOut+GrwtUp)*LeachEffAQ/WaterDepthAQ;
      if Divider<0.001 then Divider:=0.001;
      SaltAquif:=(SaltAquif+SaltIn-SaltOut)/Divider;
      if SaltAquif<0 then SaltAquif:=0.1;
    end; {for i:=1 to NrOfSteps do for k:=1 to Steplength do}
//Determining final optimimum of leaching efficiency first of 2 layers
    if (SumDiffSq<FinalDiff) then
    begin
      FinalDiff:=SumDiffSq;
      LeachEff1:=LeachEff;
    end;
  end;{for j:=1 to 100 do}

  if Count1>0 then
  begin
    SumSalt:=0;
    for i:=1 to NrOfSteps do if SaltRoot1M[i]>0 then
        SumSalt:=SumSalt+SaltRoot1M[i];
    Ave:=SumSalt/Count1;
    SumAvSq:=0;
    for i:=1 to NrOfSteps do
    begin
      if SaltRoot1M[i]>0 then
         SumAvSq:=SumAvSq+Sqr(SaltRoot1M[i]);
    end;
    Expl1:=1-sqrt(FinalDiff/SumAvSq);
  end;
end; {TwoLayer1stPart}


procedure TwoLayer2ndPart;
var i, j, k : integer;
begin
//Optimizing leaching efficiency Layer 2
  LeachEff := 0.1;
  FinalDiff := 1000000;
  for j:=1 to 180 do
  begin
    LeachEff := LeachEff+0.01;
    SumDiffSq := 0;
    SaltLayer1:=Saltlayer1i;
    Salinity:=SaltLayer2i;
//The two below can be made into one
    SaltCap:=SaltLayer2i;
    SaltCap2:=InitSaltTrans;
    SaltTrans:=InitSaltTrans;
    SaltAquif:=InitSaltAquif;

    for i:=1 to NrOfSteps do for k:=1 to StepLength do
    begin
//Salinity 1st Layer
      InitSaltR:=SaltLayer1;
      SaltIn:=(Irrig[i]*SaltIrrig+Cap2[i,k]*SaltCap)/WaterDepth1;
      SaltOut:=0.5*LeachEff1*Perc1[i,k]*SaltLayer1/WaterDepth1;
      Divider:=1+0.5*LeachEff1*Perc1[i,k]/WaterDepth1;
      if Divider<0.001 then Divider:=0.001;
      SaltLayer1:=(SaltLayer1+SaltIn-SaltOut)/Divider;
      if SaltLayer1<=0 then SaltLayer1:=0.1;
      AveSalt:=(InitSaltR+SaltLayer1)/2;
//Salinity 2nd Layer
      InitSalt2:=Salinity;
      SaltIn:=(Perc1[i,k]*AveSalt*LeachEff1+Cap3[i,k]*SaltCap2)/WaterDepth2;
      SaltOut:=0.5*LeachEff*(Perc2[i,k]+Cap2[i,k])*Salinity/WaterDepth2;
      Divider:=1+0.5*LeachEff*(Perc2[i,k]+Cap2[i,k])/WaterDepth2;
      if Divider<0.001 then Divider:=0.001;
      Salinity:=(Salinity+SaltIn-SaltOut)/Divider;
      if Salinity<=0 then Salinity:=0.1;
      SaltCap:=Salinity;
      AveSalt:=(InitSalt2+Salinity)/2;
      if (SaltRoot2M[i]>0) and (k=Steplength) then
      begin
        Diff := sqr(Salinity-SaltRoot2M[i]);
        SumDiffSq:=SumDiffSq+Diff;
        if SumDiffSq>1000 then SumDiffSq:=1000;
      end;
//Salinity transition zone
      InitSaltT:=SaltTrans;
      SaltIn:=(Perc2[i,k]*AveSalt*LeachEff2+GrwtUp*SaltAquif)/WaterDepthTR;
      SaltOut:= 0.5*(DRCf[i]*LeachEffTR*SaltTrans+GrwtDown*LeachEffTR*SaltTrans)
                /WaterDepthTR;
      Divider:=1+0.5*(DRCf[i]*LeachEffTR+GrwtDown*LeachEffTR)
               /WaterDepthTR;
      if Divider<0.001 then Divider:=0.001;
      SaltTrans:=(SaltTrans+SaltIn-SaltOut)/Divider;
      if SaltTrans<0 then SaltTrans:=0.1;
      SaltCap2:=SaltTrans;
      AveSalt:=(InitSaltT+SaltTrans)/2;
//Salinity aquifer
      SaltIn:=(GrwtDown*AveSalt*LeachEffTR+GrwtInf*SalInf)/WaterDepthAQ;
      SaltOut:=(0.5*(GrwtOut+GrwtUp)*LeachEffAQ*SaltAquif)/WaterDepthAQ;
      Divider:= 1+0.5*(GrwtOut+GrwtUp)*LeachEffAQ/WaterDepthAQ;
      if Divider<0.001 then Divider:=0.001;
      SaltAquif:=(SaltAquif+SaltIn-SaltOut)/Divider;
      if SaltAquif<0 then SaltAquif:=0.1;
    end; {for i:=1 to NrOfSteps do for k:=1 to Steplength do}
//Determining final optimimum of leaching efficiency second of 2 layers
    if (SumDiffSq<FinalDiff) then
    begin
      FinalDiff:=SumDiffSq;
      LeachEff2:=LeachEff;
    end;
  end;{for j:=1 to 100 do}
  if Count2>0 then
  begin
    SumSalt:=0;
    for i:=1 to NrOfSteps do if SaltRoot2M[i]>0 then
        SumSalt:=SumSalt+SaltRoot2M[i];
    Ave:=SumSalt/Count2;
    SumAvSq:=0;
    for i:=1 to NrOfSteps do
    begin
      if SaltRoot2M[i]>0 then
         SumAvSq:=SumAvSq+Sqr(SaltRoot2M[i]);
    end;
    Expl2:=1-sqrt(FinalDiff/SumAvSq);
  end;
end; {procedure TwoLayer2ndPart}



procedure TwoLayerTrans;
var i, j, k : integer;
begin
//Optimizing leaching efficiency Layer 2
  LeachEff := 0.1;
  FinalDiff := 1000000;
  for j:=1 to 180 do
  begin
    LeachEff := LeachEff+0.01;
    SumDiffSq := 0;
    SaltLayer1:=Saltlayer1i;
    SaltLayer2:=SaltLayer2i;
    Salinity:=InitSaltTrans;
//The two below can be made into one
    SaltCap:=SaltLayer2i;
    SaltCap2:=InitSaltTrans;
    SaltTrans:=InitSaltTrans;
    SaltAquif:=InitSaltAquif;

    for i:=1 to NrOfSteps do for k:=1 to StepLength do
    begin
//Salinity 1st Layer
      InitSaltR:=SaltLayer1;
      SaltIn:=(Irrig[i]*SaltIrrig+Cap2[i,k]*SaltCap)/WaterDepth1;
      SaltOut:=0.5*LeachEff1*Perc1[i,k]*SaltLayer1/WaterDepth1;
      Divider:=1+0.5*LeachEff1*Perc1[i,k]/WaterDepth1;
      if Divider<0.001 then Divider:=0.001;
      SaltLayer1:=(SaltLayer1+SaltIn-SaltOut)/Divider;
      if SaltLayer1<=0 then SaltLayer1:=0.1;
      AveSalt:=(InitSaltR+SaltLayer1)/2;
//Salinity 2nd Layer
      InitSalt2:=SaltLayer2;
      SaltIn:=(Perc1[i,k]*AveSalt*LeachEff1+Cap3[i,k]*SaltCap2)/WaterDepth2;
      SaltOut:=0.5*LeachEff2*(Perc2[i,k]+Cap2[i,k])*SaltLayer2/WaterDepth2;
      Divider:=1+0.5*LeachEff2*(Perc2[i,k]+Cap2[i,k])/WaterDepth2;
      if Divider<0.001 then Divider:=0.001;
      SaltLayer2:=(SaltLayer2+SaltIn-SaltOut)/Divider;
      if SaltLayer2<=0 then SaltLayer2:=0.1;
      SaltCap:=SaltLayer2;
      AveSalt:=(InitSalt2+SaltLayer2)/2;
//Salinity transition zone
      InitSaltT:=Salinity;
      SaltIn:=(Perc1[i,k]*AveSalt*LeachEff1+GrwtUp*SaltAquif)/WaterDepthTR;
      SaltOut:= 0.5*(DRCf[i]+GrwtDown)*LeachEff*Salinity/WaterDepthTR;
      Divider:=1+0.5*(DRCf[i]+GrwtDown)*LeachEff/WaterDepthTR;
      if Divider<0.001 then Divider:=0.001;
      Salinity:=(Salinity+SaltIn-SaltOut)/Divider;
      if Salinity<0 then Salinity:=0.1;
      SaltCap2:=SaltTrans;
      AveSalt:=(InitSaltT+Salinity)/2;
      if (SaltTransM[i]>0) and (k=Steplength) then
      begin
        Diff := sqr(Salinity-SaltTRansM[i]);
        SumDiffSq:=SumDiffSq+Diff;
        if SumDiffSq>1000 then SumDiffSq:=1000;
      end;
//Salinity aquifer
      SaltIn:=(GrwtDown*AveSalt+GrwtInf*SalInf)/WaterDepthAQ;
      SaltOut:=(0.5*(GrwtOut+GrwtUp)*LeachEffAQ*SaltAquif)/WaterDepthAQ;
      Divider:= 1+0.5*(GrwtOut+GrwtUp)*LeachEffAQ/WaterDepthAQ;
      if Divider<0.001 then Divider:=0.001;
      SaltAquif:=(SaltAquif+SaltIn-SaltOut)/Divider;
      if SaltAquif<0 then SaltAquif:=0.1;
    end; {for i:=1 to NrOfSteps do for k:=1 to Steplength do}
//Determining final optimimum of leaching efficiency trans. zone below 2 layers
    if (SumDiffSq<FinalDiff) then
    begin
      FinalDiff:=SumDiffSq;
      LeachEffTR:=LeachEff;
    end;
  end;{for j:=1 to 100 do}
  if Count4>0 then
  begin
    SumSalt:=0;
    for i:=1 to NrOfSteps do if SaltTransM[i]>0 then
        SumSalt:=SumSalt+SaltTransM[i];
    Ave:=SumSalt/Count4;
    SumAvSq:=0;
    for i:=1 to NrOfSteps do
    begin
      if SaltTransM[i]>0 then
         SumAvSq:=SumAvSq+Sqr(SaltTransM[i]);
    end;
    ExplTR:=1-sqrt(FinalDiff/SumAvSq);
  end;
end; {procedure TwoLayerTrans}


procedure TwoLayerAquif;
var i, j, k : integer;
begin
//Optimizing leaching efficiency Layer 2
  LeachEff := 0.1;
  FinalDiff := 1000000;
  for j:=1 to 180 do
  begin
    LeachEff := LeachEff+0.01;
    SumDiffSq := 0;
    SaltLayer1:=SaltLayer1i;;
    SaltLayer2:=SaltLayer2i;
//The two below can be made into one
    SaltCap:=SaltLayer2i;
    SaltCap2:=InitSaltTrans;
    SaltTrans:=InitSaltTrans;
    Salinity:=InitSaltAquif;

    for i:=1 to NrOfSteps do for k:=1 to StepLength do
    begin
//Salinity 1st Layer
      InitSaltR:=SaltLayer1;
      SaltIn:=(Irrig[i]*SaltIrrig+Cap2[i,k]*SaltCap)/WaterDepth1;
      SaltOut:=0.5*LeachEff1*Perc1[i,k]*SaltLayer1/WaterDepth1;
      Divider:=1+0.5*LeachEff1*Perc1[i,k]/WaterDepth1;
      if Divider<0.001 then Divider:=0.001;
      SaltLayer1:=(SaltLayer1+SaltIn-SaltOut)/Divider;
      if SaltLayer1<=0 then SaltLayer1:=0.1;
      AveSalt:=(InitSaltR+SaltLayer1)/2;
//Salinity 2nd Layer
      InitSalt2:=SaltLayer2;
      SaltIn:=(Perc1[i,k]*AveSalt*LeachEff1+Cap3[i,k]*SaltCap2)/WaterDepth2;
      SaltOut:=0.5*LeachEff2*(Perc2[i,k]+Cap2[i,k])*SaltLayer2/WaterDepth2;
      Divider:=1+0.5*LeachEff2*(Perc2[i,k]+Cap2[i,k])/WaterDepth2;
      if Divider<0.001 then Divider:=0.001;
      SaltLayer2:=(SaltLayer2+SaltIn-SaltOut)/Divider;
      if SaltLayer2<=0 then SaltLayer2:=0.1;
      SaltCap:=SaltLayer2;
      AveSalt:=(InitSalt2+SaltLayer2)/2;
//Salinity transition zone
      InitSaltT:=SaltTrans;
      SaltIn:=(Perc2[i,k]*AveSalt*LeachEff2+GrwtUp*SaltAquif*LeachEffAq)
               /WaterDepthTR;
      SaltOut:= 0.5*(DRCf[i]*LeachEffTR*SaltTrans+GrwtDown*LeachEffTR*SaltTrans)
                /WaterDepthTR;
      Divider:=1+0.5*(DRCf[i]*LeachEffTR+GrwtDown*LeachEffTR)
               /WaterDepthTR;
      if Divider<0.001 then Divider:=0.001;
      SaltTrans:=(SaltTrans+SaltIn-SaltOut)/Divider;
      if SaltTrans<0 then SaltTrans:=0.1;
      AveSalt:=(InitSaltT+SaltTrans)/2;
      SaltCap2:=SaltTrans;
//Salinity aquifer
      SaltIn:=(GrwtDown*AveSalt+GrwtInf*SalInf)/WaterDepthAQ;
      SaltOut:=(0.5*(GrwtOut+GrwtUp)*LeachEff*Salinity)/WaterDepthAQ;
      Divider:= 1+0.5*(GrwtOut+GrwtUp)*LeachEffAQ/WaterDepthAQ;
      if Divider<0.001 then Divider:=0.001;
      Salinity:=(Salinity+SaltIn-SaltOut)/Divider;
      if Salinity<0 then Salinity:=0.1;
      if (SaltAquifM[i]>0) and (k=Steplength) then
      begin
        Diff := sqr(Salinity-SaltAquifM[i]);
        SumDiffSq:=SumDiffSq+Diff;
        if SumDiffSq>1000 then SumDiffSq:=1000;
      end;
    end; {for i:=1 to NrOfSteps do for k:=1 to Steplength do}
//Determining final optimimum of leaching efficiency aquifer below 2 layers
    if (SumDiffSq<FinalDiff) then
    begin
      FinalDiff:=SumDiffSq;
      LeachEffAQ:=LeachEff;
    end;
  end;{for j:=1 to 180 do}
  if Count5>0 then
  begin
    SumSalt:=0;
    for i:=1 to NrOfSteps do if SaltAquifM[i]>0 then
        SumSalt:=SumSalt+SaltAquifM[i];
    Ave:=SumSalt/Count5;
    SumAvSq:=0;
    for i:=1 to NrOfSteps do
    begin
      if SaltAquifM[i]>0 then
         SumAvSq:=SumAvSq+Sqr(SaltAquifM[i]);
    end;
    ExplAQ:=1-sqrt(FinalDiff/SumAvSq);
  end;
end; {procedure TwoLayerAquif}



procedure ThreeLayer1stPart;
var i, j, k : integer;
begin
//Optimizing leaching efficiency Layer 1
  LeachEff := 0.1;
  FinalDiff := 1000000;
  for j:=1 to 180 do
  begin
    LeachEff := LeachEff+0.01;
    SumDiffSq := 0;
    Salinity:=SaltLayer1i;
    SaltLayer2:=SaltLayer2i;
    SaltLayer3:=SaltLayer3i;
//The two below can be made into one
    SaltCap:=SaltLayer2i;
    SaltCap2:=SaltLayer3i;
    SaltCap3:=InitSaltTrans;
    SaltTrans:=InitSaltTrans;
    SaltAquif:=InitSaltAquif;
    for i:=1 to NrOfSteps do for k:=1 to StepLength do
    begin
//Salinity 1st Layer
      InitSaltR:=Salinity;
      SaltIn:=(Irrig[i]*SaltIrrig+Cap2[i,k]*SaltCap)/WaterDepth1;
      SaltOut:=0.5*LeachEff*Perc1[i,k]*Salinity/WaterDepth1;
      Divider:=1+0.5*LeachEff*Perc1[i,k]/WaterDepth1;
      if Divider<0.001 then Divider:=0.001;
      Salinity:=(Salinity+SaltIn-SaltOut)/Divider;
      if Salinity<=0 then Salinity:=0.1;
      if (SaltRoot1M[i]>0) and (k=StepLength) then
      begin
        Diff := abs(Salinity-SaltRoot1M[i]);
        SumDiffSq:=SumDiffSq+Diff;
        if SumDiffSq>1000 then SumDiffSq:=1000;
      end;
      AveSalt:=(InitSaltR+Salinity)/2;
//Salinity 2nd Layer
      InitSalt2:=SaltLayer2;
      SaltIn:=(Perc1[i,k]*AveSalt*LeachEff+Cap3[i,k]*SaltCap2)/WaterDepth2;
      SaltOut:=0.5*LeachEff2*(Perc2[i,k]+Cap2[i,k])*SaltLayer2/WaterDepth2;
      Divider:=1+0.5*LeachEff2*(Perc2[i,k]+Cap2[i,k])/WaterDepth2;
      if Divider<0.001 then Divider:=0.001;
      SaltLayer2:=(SaltLayer2+SaltIn-SaltOut)/Divider;
      if SaltLayer2<=0 then SaltLayer2:=0.1;
      SaltCap:=SaltLayer2;
      AveSalt:=(InitSalt2+SaltLayer2)/2;
//Salinity 3rd layer
      InitSalt3:=SaltLayer3;
      SaltIn:=(Perc2[i,k]*AveSalt*LeachEff2+Cap4[i,k]*SaltCap3)/WaterDepth3;
      SaltOut:=0.5*LeachEff3*(Perc3[i,k]+Cap3[i,k])*SaltLayer3/WaterDepth3;
      Divider:=1+0.5*LeachEff3*(Perc3[i,k]+Cap3[i,k])/WaterDepth3;
      if Divider<0.001 then Divider:=0.001;
      SaltLayer3:=(SaltLayer3+SaltIn-SaltOut)/Divider;
      if SaltLayer3<0 then SaltLayer3:=0.1;
      SaltCap2:=SaltLayer3;
      AveSalt:=(InitSalt3+SaltLayer3)/2;
//Salinity transition zone
      InitSaltT:=SaltTrans;
      SaltIn:=(Perc3[i,k]*AveSalt*LeachEff3+GrwtUp*SaltAquif)/WaterDepthTR;
      SaltOut:= 0.5*(DRCf[i]*LeachEffTR*SaltTrans+GrwtDown*LeachEffTR*SaltTrans)
                /WaterDepthTR;
      Divider:=1+0.5*(DRCf[i]*LeachEffTR+GrwtDown*LeachEffTR)
               /WaterDepthTR;
      if Divider<0.001 then Divider:=0.001;
      SaltTrans:=(SaltTrans+SaltIn-SaltOut)/Divider;
      if SaltTrans<0 then SaltTrans:=0.1;
      SaltCap3:=SaltTrans;
      AveSalt:=(InitSaltT+SaltTrans)/2;
//Salinity aquifer
      SaltIn:=(GrwtDown*AveSalt+GrwtInf*SalInf)/WaterDepthAQ;
      SaltOut:=(0.5*(GrwtOut+GrwtUp)*LeachEffAQ*SaltAquif)/WaterDepthAQ;
      Divider:= 1+0.5*(GrwtOut+GrwtUp)*LeachEffAQ/WaterDepthAQ;
      if Divider<0.001 then Divider:=0.001;
      SaltAquif:=(SaltAquif+SaltIn-SaltOut)/Divider;
      if SaltAquif<0 then SaltAquif:=0.1;
    end; {for i:=1 to NrOfSteps do for k:=1 to Steplength do}
//Determining final optimimum of leaching efficiency first of 3 layers
    if (SumDiffSq<FinalDiff) then
    begin
      FinalDiff:=SumDiffSq;
      LeachEff1:=LeachEff;
    end;
  end;{for j:=1 to 180 do}
  if Count1>0 then
  begin
    SumSalt:=0;
    for i:=1 to NrOfSteps do if SaltRoot1M[i]>0 then
        SumSalt:=SumSalt+SaltRoot1M[i];
    Ave:=SumSalt/Count1;
    SumAvSq:=0;
    for i:=1 to NrOfSteps do
    begin
      if SaltRoot1M[i]>0 then
         SumAvSq:=SumAvSq+Sqr(SaltRoot1M[i]);
    end;
//    Expl1:=1-sqrt(FinalDiff/SumAvSq);
    Expl1:=1-FinalDiff/SumSalt;
  end;
end; {ThreeLayer1stPart;}



procedure ThreeLayer2ndPart;
var i, j, k : integer;
begin
//Optimizing leaching efficiency Layer 2
  LeachEff := 0.1;
  FinalDiff := 1000000;
  for j:=1 to 180 do
  begin
    LeachEff := LeachEff+0.01;
    SumDiffSq := 0;
    SaltLayer1:=SaltLayer1i;
    SaltLayer2:=SaltLayer2i;
    SaltLayer3:=SaltLayer3i;
    Salinity:=SaltLayer2i;
//The two below can be made into one
    SaltCap:=SaltLayer2i;
    SaltCap2:=SaltLayer3i;
    SaltCap3:=InitSaltTrans;
    SaltTrans:=InitSaltTrans;
    SaltAquif:=InitSaltAquif;

    for i:=1 to NrOfSteps do for k:=1 to StepLength do
    begin
//Salinity 1st Layer
      InitSaltR:=SaltLayer1;
      SaltIn:=(Irrig[i]*SaltIrrig+Cap2[i,k]*SaltCap)/WaterDepth1;
      SaltOut:=0.5*LeachEff1*Perc1[i,k]*SaltLayer1/WaterDepth1;
      Divider:=1+0.5*LeachEff1*Perc1[i,k]/WaterDepth1;
      if Divider<0.001 then Divider:=0.001;
      SaltLayer1:=(SaltLayer1+SaltIn-SaltOut)/Divider;
      if SaltLayer1<=0 then SaltLayer1:=0.1;
      AveSalt:=(InitSaltR+SaltLayer1)/2;
//Salinity 2nd Layer
      InitSalt2:=Salinity;
      SaltIn:=(Perc1[i,k]*AveSalt*LeachEff1+Cap3[i,k]*SaltCap2)/WaterDepth2;
      SaltOut:=0.5*LeachEff*(Perc2[i,k]+Cap2[i,k])*Salinity/WaterDepth2;
      Divider:=1+0.5*LeachEff*(Perc2[i,k]+Cap2[i,k])/WaterDepth2;
      if Divider<0.001 then Divider:=0.001;
      Salinity:=(Salinity+SaltIn-SaltOut)/Divider;
      if Salinity<=0 then Salinity:=0.1;
      SaltCap:=Salinity;
      AveSalt:=(InitSalt2+Salinity)/2;
      if (SaltRoot2M[i]>0) and (k=StepLength) then
      begin
        Diff := sqr(Salinity-SaltRoot2M[i]);
        SumDiffSq:=SumDiffSq+Diff;
        if SumDiffSq>1000 then SumDiffSq:=1000;
      end;
//Salinity 3rd layer
      InitSalt3:=SaltLayer3;
      SaltIn:=(Perc2[i,k]*AveSalt*LeachEff2+Cap4[i,k]*SaltCap3)/WaterDepth3;
      SaltOut:=0.5*LeachEff3*(Perc3[i,k]+Cap3[i,k])*SaltLayer3/WaterDepth3;
      Divider:=1+0.5*LeachEff3*(Perc3[i,k]+Cap3[i,k])/WaterDepth3;
      if Divider<0.001 then Divider:=0.001;
      SaltLayer3:=(SaltLayer3+SaltIn-SaltOut)/Divider;
      if SaltLayer3<0 then SaltLayer3:=0.1;
      SaltCap2:=SaltLayer3;
      AveSalt:=(InitSalt3+SaltLayer3)/2;
//Salinity transition zone
      InitSaltT:=SaltTrans;
      SaltIn:=(Perc3[i,k]*AveSalt*LeachEff3+GrwtUp*SaltAquif)/WaterDepthTR;
      SaltOut:= 0.5*(DRCf[i]*LeachEffTR*SaltTrans+GrwtDown*LeachEffTR*SaltTrans)
                /WaterDepthTR;
      Divider:=1+0.5*(DRCf[i]*LeachEffTR+GrwtDown*LeachEffTR)
               /WaterDepthTR;
      if Divider<0.001 then Divider:=0.001;
      SaltTrans:=(SaltTrans+SaltIn-SaltOut)/Divider;
      if SaltTrans<0 then SaltTrans:=0.1;
      SaltCap3:=SaltTrans;
      AveSalt:=(InitSaltT+SaltTrans)/2;
//Salinity aquifer
      SaltIn:=(GrwtDown*AveSalt+GrwtInf*SalInf)/WaterDepthAQ;
      SaltOut:=(0.5*(GrwtOut+GrwtUp)*LeachEffAQ*SaltAquif)/WaterDepthAQ;
      Divider:= 1+0.5*(GrwtOut+GrwtUp)*LeachEffAQ/WaterDepthAQ;
      if Divider<0.001 then Divider:=0.001;
      SaltAquif:=(SaltAquif+SaltIn-SaltOut)/Divider;
      if SaltAquif<0 then SaltAquif:=0.1;
    end; {for i:=1 to NrOfSteps do for k:=1 to Steplength do}
//Determining final optimimum of leaching efficiency second of 3 layers
    if (SumDiffSq<FinalDiff) then
    begin
      FinalDiff:=SumDiffSq;
      LeachEff2:=LeachEff;
    end;
  end;{for j:=1 to 100 do}
  if Count2>0 then
  begin
    SumSalt:=0;
    for i:=1 to NrOfSteps do if SaltRoot2M[i]>0 then
        SumSalt:=SumSalt+SaltRoot2M[i];
    Ave:=SumSalt/Count2;
    SumAvSq:=0;
    for i:=1 to NrOfSteps do
    begin
      if SaltRoot2M[i]>0 then
         SumAvSq:=SumAvSq+Sqr(SaltRoot2M[i]);
    end;
    Expl2:=1-sqrt(FinalDiff/SumAvSq);
  end;
end; {ThreeLayer2ndPart;}



procedure ThreeLayer3rdPart;
var i, j, k : integer;
begin
//Optimizing leaching efficiency Layer 3
  LeachEff := 0.1;
  FinalDiff := 1000000;
  for j:=1 to 180 do
  begin
    LeachEff := LeachEff+0.01;
    SumDiffSq := 0;
    SaltLayer1:=SaltLayer1i;
    SaltLayer2:=SaltLayer2i;
    SaltLayer3:=SaltLayer3i;
    Salinity:=SaltLayer3i;
//The two below can be made into one
    SaltCap:=SaltLayer2i;
    SaltCap2:=SaltLayer3i;
    SaltCap3:=InitSaltTrans;
    SaltTrans:=InitSaltTrans;
    SaltAquif:=InitSaltAquif;

    for i:=1 to NrOfSteps do for k:=1 to StepLength do
    begin
//Salinity 1st Layer
      InitSaltR:=SaltLayer1;
      SaltIn:=(Irrig[i]*SaltIrrig+Cap2[i,k]*SaltCap)/WaterDepth1;
      SaltOut:=0.5*LeachEff1*Perc1[i,k]*SaltLayer1/WaterDepth1;
      Divider:=1+0.5*LeachEff1*Perc1[i,k]/WaterDepth1;
      if Divider<0.001 then Divider:=0.001;
      SaltLayer1:=(SaltLayer1+SaltIn-SaltOut)/Divider;
      if SaltLayer1<=0 then SaltLayer1:=0.1;
      AveSalt:=(InitSaltR+SaltLayer1)/2;
//Salinity 2nd Layer
      InitSalt2:=SaltLayer2;
      SaltIn:=(Perc1[i,k]*AveSalt*LeachEff1+Cap3[i,k]*SaltCap2)/WaterDepth2;
      SaltOut:=0.5*LeachEff2*(Perc2[i,k]+Cap2[i,k])*SaltLayer2/WaterDepth2;
      Divider:=1+0.5*LeachEff2*(Perc2[i,k]+Cap2[i,k])/WaterDepth2;
      if Divider<0.001 then Divider:=0.001;
      SaltLayer2:=(SaltLayer2+SaltIn-SaltOut)/Divider;
      if SaltLayer2<=0 then SaltLayer2:=0.1;
      SaltCap:=SaltLayer2;
      AveSalt:=(InitSalt2+SaltLayer2)/2;
//Salinity 3rd layer
      InitSalt3:=Salinity;
      SaltIn:=(Perc2[i,k]*AveSalt*LeachEff2+Cap4[i,k]*SaltCap3)/WaterDepth3;
      SaltOut:=0.5*LeachEff*(Perc3[i,k]+Cap3[i,k])*Salinity/WaterDepth3;
      Divider:=1+0.5*LeachEff*(Perc3[i,k]+Cap3[i,k])/WaterDepth3;
      if Divider<0.001 then Divider:=0.001;
      Salinity:=(Salinity+SaltIn-SaltOut)/Divider;
      if Salinity<0 then Salinity:=0.1;
      SaltCap2:=Salinity;
      AveSalt:=(InitSalt3+Salinity)/2;
      if (SaltRoot3M[i]>0) and (k=StepLength) then
      begin
        Diff := sqr(Salinity-SaltRoot3M[i]);
        SumDiffSq:=SumDiffSq+Diff;
        if SumDiffSq>1000 then SumDiffSq:=1000;
      end;
//Salinity transition zone
      InitSaltT:=SaltTrans;
      SaltIn:=(Perc3[i,k]*AveSalt*LeachEff+GrwtUp*SaltAquif)/WaterDepthTR;
      SaltOut:= 0.5*(DRCf[i]*LeachEffTR*SaltTrans+GrwtDown*LeachEffTR*SaltTrans)
                /WaterDepthTR;
      Divider:=1+0.5*(DRCf[i]*LeachEffTR+GrwtDown*LeachEffTR)
               /WaterDepthTR;
      if Divider<0.001 then Divider:=0.001;
      SaltTrans:=(SaltTrans+SaltIn-SaltOut)/Divider;
      if SaltTrans<0 then SaltTrans:=0.1;
      SaltCap3:=SaltTrans;
      AveSalt:=(InitSaltT+SaltTrans)/2;
//Salinity aquifer
      SaltIn:=(GrwtDown*AveSalt+GrwtInf*SalInf)/WaterDepthAQ;
      SaltOut:=(0.5*(GrwtOut+GrwtUp)*LeachEffAQ*SaltAquif)/WaterDepthAQ;
      Divider:= 1+0.5*(GrwtOut+GrwtUp)*LeachEffAQ/WaterDepthAQ;
      if Divider<0.001 then Divider:=0.001;
      SaltAquif:=(SaltAquif+SaltIn-SaltOut)/Divider;
      if SaltAquif<0 then SaltAquif:=0.1;
    end; {for i:=1 to NrOfSteps do for k:=1 to Steplength do}
//Determining final optimimum of leaching efficiency third of 3 layers
    if (SumDiffSq<FinalDiff) then
    begin
      FinalDiff:=SumDiffSq;
      LeachEff3:=LeachEff;
    end;
  end;{for j:=1 to 100 do}
  if Count3>0 then
  begin
    SumSalt:=0;
    for i:=1 to NrOfSteps do if SaltRoot3M[i]>0 then
        SumSalt:=SumSalt+SaltRoot3M[i];
    Ave:=SumSalt/Count3;
    SumAvSq:=0;
    for i:=1 to NrOfSteps do
    begin
      if SaltRoot3M[i]>0 then
         SumAvSq:=SumAvSq+Sqr(SaltRoot3M[i]);
    end;
    Expl3:=1-sqrt(FinalDiff/SumAvSq);
  end;
end; {ThreeLayer3rdPart;}



procedure ThreeLayerTrans;
var i, j, k : integer;
begin
//Optimizing leaching efficiency Transition zone
  LeachEff := 0.1;
  FinalDiff := 1000000;
  for j:=1 to 180 do
  begin
    LeachEff := LeachEff+0.01;
    SumDiffSq := 0;
    SaltLayer1:=SaltLayer1i;
    SaltLayer2:=SaltLayer2i;
    SaltLayer3:=SaltLayer3i;
//The two below can be made into one
    SaltCap:=SaltLayer2i;
    SaltCap2:=SaltLayer3i;
    SaltCap3:=InitSaltTrans;
    SaltTrans:=InitSaltTrans;
    Salinity:=InitSaltTrans;
    SaltAquif:=InitSaltAquif;

    for i:=1 to NrOfSteps do for k:=1 to StepLength do
    begin
//Salinity 1st Layer
      InitSaltR:=SaltLayer1;
      SaltIn:=(Irrig[i]*SaltIrrig+Cap2[i,k]*SaltCap)/WaterDepth1;
      SaltOut:=0.5*LeachEff1*Perc1[i,k]*SaltLayer1/WaterDepth1;
      Divider:=1+0.5*LeachEff1*Perc1[i,k]/WaterDepth1;
      if Divider<0.001 then Divider:=0.001;
      SaltLayer1:=(SaltLayer1+SaltIn-SaltOut)/Divider;
      if SaltLayer1<=0 then SaltLayer1:=0.1;
      AveSalt:=(InitSaltR+SaltLayer1)/2;
//Salinity 2nd Layer
      InitSalt2:=SaltLayer2;
      SaltIn:=(Perc1[i,k]*AveSalt*LeachEff1+Cap3[i,k]*SaltCap2)/WaterDepth2;
      SaltOut:=0.5*LeachEff2*(Perc2[i,k]+Cap2[i,k])*SaltLayer2/WaterDepth2;
      Divider:=1+0.5*LeachEff2*(Perc2[i,k]+Cap2[i,k])/WaterDepth2;
      if Divider<0.001 then Divider:=0.001;
      SaltLayer2:=(SaltLayer2+SaltIn-SaltOut)/Divider;
      if SaltLayer2<=0 then SaltLayer2:=0.1;
      AveSalt:=(InitSalt2+SaltLayer2)/2;
      SaltCap:=SaltLayer2;
//Salinity 3rd layer
      InitSalt3:=SaltLayer3;
      SaltIn:=(Perc2[i,k]*AveSalt*LeachEff2+Cap4[i,k]*SaltCap3)/WaterDepth3;
      SaltOut:=0.5*LeachEff3*(Perc3[i,k]+Cap3[i,k])*SaltLayer3/WaterDepth3;
      Divider:=1+0.5*LeachEff3*(Perc3[i,k]+Cap3[i,k])/WaterDepth3;
      if Divider<0.001 then Divider:=0.001;
      SaltLayer3:=(SaltLayer3+SaltIn-SaltOut)/Divider;
      if SaltLayer3<0 then SaltLayer3:=0.1;
      SaltCap2:=SaltLayer3;
      AveSalt:=(InitSalt3+SaltLayer3)/2;
//Salinity transition zone
      InitSaltT:=Salinity;
      SaltIn:=(Perc3[i,k]*AveSalt*LeachEff3+GrwtUp*SaltAquif)/WaterDepthTR;
      SaltOut:= 0.5*(DRCf[i]*LeachEff*Salinity+GrwtDown*LeachEff*Salinity)
                /WaterDepthTR;
      Divider:=1+0.5*(DRCf[i]*LeachEff+GrwtDown*LeachEff)
               /WaterDepthTR;
      if Divider<0.001 then Divider:=0.001;
      Salinity:=(Salinity+SaltIn-SaltOut)/Divider;
      if Salinity<0 then Salinity:=0.1;
      SaltCap3:=Salinity;
      AveSalt:=(InitSaltT+Salinity)/2;
      if (SaltTransM[i]>0) and (k=StepLength) then
      begin
        Diff := sqr(Salinity-SaltTransM[i]);
        SumDiffSq:=SumDiffSq+Diff;
        if SumDiffSq>1000 then SumDiffSq:=1000;
      end;
//Salinity aquifer
      SaltIn:=(GrwtDown*AveSalt+GrwtInf*SalInf)/WaterDepthAQ;
      SaltOut:=0.5*(GrwtOut+GrwtUp)*LeachEffAQ*SaltAquif/WaterDepthAQ;
      Divider:= 1+0.5*(GrwtOut+GrwtUp)*LeachEffAQ/WaterDepthAQ;
      if Divider<0.001 then Divider:=0.001;
      SaltAquif:=(SaltAquif+SaltIn-SaltOut)/Divider;
      if SaltAquif<0 then SaltAquif:=0.1;
    end; {for i:=1 to NrOfSteps do for k:=1 to Steplength do}
//Determining final optimimum of leaching efficiency trans. zone below 3 layers
    if (SumDiffSq<FinalDiff) then
    begin
      FinalDiff:=SumDiffSq;
      LeachEffTR:=LeachEff;
    end;
  end;{for j:=1 to 180 do}
  if Count4>0 then
  begin
    SumSalt:=0;
    for i:=1 to NrOfSteps do if SaltTransM[i]>0 then
        SumSalt:=SumSalt+SaltTransM[i];
    Ave:=SumSalt/Count4;
    SumAvSq:=0;
    for i:=1 to NrOfSteps do
    begin
      if SaltTransM[i]>0 then
         SumAvSq:=SumAvSq+Sqr(SaltTransM[i]);
    end;
    ExplTR:=1-sqrt(FinalDiff/SumAvSq);
  end;
end; {ThreeLayerTrans;}



procedure ThreeLayerAquif;
var i, j, k : integer;
begin
//Optimizing leaching efficiency aquifer
  LeachEff := 0.1;
  FinalDiff := 1000000;
  for j:=1 to 180 do
  begin
    LeachEff := LeachEff+0.01;
    SumDiffSq := 0;
    SaltLayer1:=SaltLayer1i;
    SaltLayer2:=SaltLayer2i;
    SaltLayer3:=SaltLayer2i;
//The two below can be made into one
    SaltCap:=SaltLayer2i;
    SaltCap2:=SaltLayer3i;
    SaltCap3:=InitSaltTrans;
    SaltTrans:=InitSaltTrans;
    SaltAquif:=InitSaltAquif;
    Salinity:=InitSaltAquif;

    for i:=1 to NrOfSteps do for k:=1 to StepLength do
    begin
//Salinity 1st Layer
      InitSaltR:=SaltLayer1;
      SaltIn:=(Irrig[i]*SaltIrrig+Cap2[i,k]*SaltCap)/WaterDepth1;
      SaltOut:=0.5*LeachEff1*Perc1[i,k]*SaltLayer1/WaterDepth1;
      Divider:=1+0.5*LeachEff1*Perc1[i,k]/WaterDepth1;
      if Divider<0.001 then Divider:=0.001;
      SaltLayer1:=(SaltLayer1+SaltIn-SaltOut)/Divider;
      if SaltLayer1<=0 then SaltLayer1:=0.1;
      AveSalt:=(InitSaltR+SaltLayer1)/2;
//Salinity 2nd Layer
      InitSalt2:=SaltLayer2;
      SaltIn:=(Perc1[i,k]*AveSalt*LeachEff1+Cap3[i,k]*SaltCap2)/WaterDepth2;
      SaltOut:=0.5*LeachEff2*(Perc2[i,k]+Cap2[i,k])*SaltLayer2/WaterDepth2;
      Divider:=1+0.5*LeachEff2*(Perc2[i,k]+Cap2[i,k])/WaterDepth2;
      if Divider<0.001 then Divider:=0.001;
      SaltLayer2:=(SaltLayer2+SaltIn-SaltOut)/Divider;
      if SaltLayer2<=0 then SaltLayer2:=0.1;
      SaltCap:=SaltLayer2;
      AveSalt:=(InitSalt2+SaltLayer2)/2;
//Salinity 3rd layer
      InitSalt3:=SaltLayer3;
      SaltIn:=(Perc2[i,k]*AveSalt*LeachEff2+Cap4[i,k]*SaltCap3)/WaterDepth3;
      SaltOut:=0.5*LeachEff3*(Perc3[i,k]+Cap3[i,k])*SaltLayer3/WaterDepth3;
      Divider:=1+0.5*LeachEff3*(Perc3[i,k]+Cap3[i,k])/WaterDepth3;
      if Divider<0.001 then Divider:=0.001;
      SaltLayer3:=(SaltLayer3+SaltIn-SaltOut)/Divider;
      if SaltLayer3<0 then SaltLayer3:=0.1;
      SaltCap2:=SaltLayer3;
      AveSalt:=(InitSalt3+SaltLayer3)/2;
//Salinity transition zone
      InitSaltT:=SaltTrans;
      SaltIn:=(Perc3[i,k]*AveSalt*LeachEff3+GrwtUp*SaltAquif)/WaterDepthTR;
      SaltOut:= 0.5*(DRCf[i]*LeachEffTR*SaltTrans+GrwtDown*LeachEffTR*SaltTrans)
               /WaterDepthTR;
      Divider:=1+0.5*(DRCf[i]*LeachEffTR+GrwtDown*LeachEffTR)
               /WaterDepthTR;
      if Divider<0.001 then Divider:=0.001;
      SaltTrans:=(SaltTrans+SaltIn-SaltOut)/Divider;
      if SaltTrans<0 then SaltTrans:=0.1;
      SaltCap3:=SaltTrans;
      AveSalt:=(InitSaltT+SaltTrans)/2;
//Salinity aquifer
      SaltIn:=(GrwtDown*AveSalt+GrwtInf*SalInf)/WaterDepthAQ;
      SaltOut:=(0.5*(GrwtOut+GrwtUp)*LeachEff*Salinity)/WaterDepthAQ;
      Divider:= 1+0.5*(GrwtOut+GrwtUp)*LeachEffAQ/WaterDepthAQ;
      if Divider<0.001 then Divider:=0.001;
      Salinity:=(Salinity+SaltIn-SaltOut)/Divider;
      if Salinity<0 then Salinity:=0.1;
      if (SaltAquifM[i]>0) and (k=StepLength) then
      begin
        Diff := sqr(Salinity-SaltAquifM[i]);
        SumDiffSq:=SumDiffSq+Diff;
        if SumDiffSq>1000 then SumDiffSq:=1000;
      end;
    end; {for i:=1 to NrOfSteps do for k:=1 to Steplength do}
//Determining final optimimum of leaching efficiency aquifer below 3 layers
    if (SumDiffSq<FinalDiff) then
    begin
      FinalDiff:=SumDiffSq;
      LeachEffAQ:=LeachEff;
    end;
  end;{for j:=1 to 100 do}
  if Count5>0 then
  begin
    SumSalt:=0;
    for i:=1 to NrOfSteps do if SaltAquifM[i]>0 then
        SumSalt:=SumSalt+SaltAquifM[i];
    Ave:=SumSalt/Count5;
    SumAvSq:=0;
    for i:=1 to NrOfSteps do
    begin
      if SaltAquifM[i]>0 then
         SumAvSq:=SumAvSq+Sqr(SaltAquifM[i]);
    end;
    ExplAQ:=1-sqrt(FinalDiff/SumAvSq);
  end;
end; {ThreeLayerAquif;}



procedure SaltBalance;
var i, j : integer;
    SumDiffSq1, SumDiffSq2, SumDiffSq3, SumDiffSq4, SumDiffSq5 : real;

begin
  SumDiffSq1:=0;
  SumDiffSq2:=0;
  SumDiffSq3:=0;
  SumDiffSq4:=0;
  SumDiffSq5:=0;
  if OneLayer then SaltCap:=InitSaltTrans;
  if TwoLayer then
  begin
    SaltCap:=SaltLayer2i;
    SaltCap2:=InitSaltTrans;
  end;
  if ThreeLayer then
  begin
    SaltCap:=SaltLayer2i;
    SaltCap2:=SaltLayer3i;
    SaltCap3:=InitSaltTrans;
  end;
  SaltLayer1:=SaltLayer1i;
  SaltLayer2:=SaltLayer2i;
  SaltLayer3:=SaltLayer3i;
  SaltTrans:=InitSaltTrans;
  SaltAquif:=InitSaltAquif;

  for i:=1 to NrOfSteps do              {Start calculations for all times teps}
  begin

    for j:=1 to StepLength do              {Calculations whithin the time step}
    begin
//Final salinity if there is one layer in the rootzone
      if OneLayer then
      begin
//Salinity rootzone
        InitSaltR:=SaltLayer1;
        SaltIn:=(Irrig[i]*SaltIrrig+Cap1[i,j]*SaltCap)/WaterDepth1;
        SaltOut:=0.5*LeachEff1*Perc1[i,j]*SaltLayer1/WaterDepth1;
        Divider:=1+0.5*LeachEff1*Perc1[i,j]/WaterDepth1;
        if Divider<0.001 then Divider:=0.001;
        SaltLayer1:=(SaltLayer1+SaltIn-SaltOut)/Divider;
        if SaltLayer1<=0 then SaltLayer1:=0.1;
        if (SaltRoot1M[i]>0) and (j=Steplength) then
        begin
          Diff := sqr(SaltLayer1-SaltRoot1M[i]);
          SumDiffSq1:=SumDiffSq1+Diff;
        end;
        AveSalt:=(InitSaltR+SaltLayer1)/2;
//Salinity transition zone
        InitSaltT:=SaltTrans;
        SaltIn:=(LeachEff1*Perc1[i,j]*AveSalt+GrwtUp*SaltAquif)/WaterDepthTR;
        SaltOut:=
            0.5*LeachEffTR*(DRCf[i]*SaltTrans-GrwtInf*SaltTrans)/WaterDepthTR;
        Divider:=1+0.5*(DRCf[i]*LeachEffTR-GrwtInf*LeachEffTR)
                 /WaterDepthTR;
        if Divider<0.001 then Divider:=0.001;
        SaltTrans:=(SaltTrans+SaltIn-SaltOut)/Divider;
        if SaltTrans<0 then SaltTrans:=0.1;
        if (SaltTransM[i]>0) and (j=Steplength) then
        begin
          Diff := sqr(SaltTrans-SaltTransM[i]);
          SumDiffSq4:=SumDiffSq4+Diff;
        end;
        SaltCap:=SaltTrans;
        AveSalt:=(InitSaltT+SaltTrans)/2;
//Salinity aquifer
        SaltIn:=(LeachEffTR*GrwtDown*AveSalt+GrwtInf*SalInf)/WaterDepthAQ;
        SaltOut:=0.5*LeachEffAQ*(GrwtOut+GrwtUp)*SaltAquif/WaterDepthAQ;
        Divider:= 1+0.5*(GrwtOut+GrwtUp)*LeachEffAQ/WaterDepthAQ;
        if Divider<0.001 then Divider:=0.001;
        SaltAquif:=(SaltAquif+SaltIn-SaltOut)/Divider;
        if SaltAquif<0 then SaltAquif:=0.1;
        if (SaltAquifM[i]>0) and (j=Steplength) then
        begin
          Diff := sqr(SaltAquif-SaltAquifM[i]);
          SumDiffSq5:=SumDiffSq5+Diff;
        end;
      end; {if OneLayer then}

//Final salinity if there are two layers in the rootzone
      if TwoLayer then
      begin
//Salinity 1st Layer
        InitSaltR:=SaltLayer1;
        SaltIn:=(Irrig[i]*SaltIrrig+Cap2[i,j]*SaltCap)/WaterDepth1;
        SaltOut:=0.5*LeachEff1*Perc1[i,j]*SaltLayer1/WaterDepth1;
        Divider:=1+0.5*LeachEff1*Perc1[i,j]/WaterDepth1;
        if Divider<0.001 then Divider:=0.001;
        SaltLayer1:=(SaltLayer1+SaltIn-SaltOut)/Divider;
        if SaltLayer1<=0 then SaltLayer1:=0.1;
        AveSalt:=(InitSaltR+SaltLayer1)/2;
        if (SaltRoot1M[i]>0) and (j=Steplength) then
        begin
          Diff := sqr(SaltLayer1-SaltRoot1M[i]);
          SumDiffSq1:=SumDiffSq1+Diff;
        end;
//Salinity 2nd Layer
        InitSalt2:=SaltLayer2;
        SaltIn:=(LeachEff1*Perc1[i,j]*AveSalt+Cap3[i,j]*SaltCap2)/WaterDepth2;
        SaltOut:=0.5*LeachEff2*(Perc2[i,j]+Cap2[i,j])*SaltLayer2/WaterDepth2;
        Divider:=1+0.5*LeachEff2*(Perc2[i,j]+Cap2[i,j])/WaterDepth2;
        if Divider<0.001 then Divider:=0.001;
        SaltLayer2:=(SaltLayer2+SaltIn-SaltOut)/Divider;
        if SaltLayer2<=0 then SaltLayer2:=0.1;
        if (SaltRoot2M[i]>0) and (j=Steplength) then
        begin
          Diff := sqr(SaltLayer2-SaltRoot2M[i]);
          SumDiffSq2:=SumDiffSq1+Diff;
        end;
        SaltCap:=SaltLayer2;
        AveSalt:=(InitSalt2+SaltLayer2)/2;
//Salinity transition zone
        InitSaltT:=SaltTrans;
        SaltIn:=(LeachEff2*Perc2[i,j]*AveSalt+GrwtUp*SaltAquif)/WaterDepthTR;
        SaltOut:=0.5*LeachEffTR*SaltTrans*DRCf[i]/WaterDepthTR;
        Divider:=1+0.5*(DRCf[i]*LeachEffTR)/WaterDepthTR;
        if Divider<0.001 then Divider:=0.001;
        SaltTrans:=(SaltTrans+SaltIn-SaltOut)/Divider;
        if SaltTrans<0 then SaltTrans:=0.1;
        if (SaltTransM[i]>0) and (j=Steplength) then
        begin
          Diff := sqr(SaltTrans-SaltTransM[i]);
          SumDiffSq4:=SumDiffSq4+Diff;
        end;
        SaltCap2:=SaltTrans;
        AveSalt:=(InitSaltT+SaltTrans)/2;
//Salinity aquifer
        SaltIn:=(LeachEffTR*GrwtDown*AveSalt+GrwtInf*SalInf)/WaterDepthAQ;
        SaltOut:=(0.5*(GrwtOut+GrwtUp)*LeachEffAQ*SaltAquif)/WaterDepthAQ;
        Divider:= 1+0.5*(GrwtOut+GrwtUp)*LeachEffAQ/WaterDepthAQ;
        SaltAquif:=(SaltAquif+SaltIn-SaltOut)/Divider;
        if SaltAquif<0 then SaltAquif:=0.1;
        if (SaltAquifM[i]>0) and (j=Steplength) then
        begin
          Diff := sqr(SaltAquif-SaltAquifM[i]);
          SumDiffSq5:=SumDiffSq5+Diff;
        end;
      end; {if TwoLayer then}
      
//Final salinity if there are three layers in the rootzone
      if ThreeLayer then
      begin
//Salinity 1st Layer
        InitSaltR:=SaltLayer1;
        SaltIn:=(Irrig[i]*SaltIrrig+Cap2[i,j]*SaltCap)/WaterDepth1;
        SaltOut:=0.5*LeachEff1*Perc1[i,j]*SaltLayer1/WaterDepth1;
        Divider:=1+0.5*LeachEff1*Perc1[i,j]/WaterDepth1;
        if Divider<0.001 then Divider:=0.001;
        SaltLayer1:=(SaltLayer1+SaltIn-SaltOut)/Divider;
        if SaltLayer1<=0 then SaltLayer1:=0.1;
        if (SaltRoot1M[i]>0) and (j=Steplength) then
        begin
          Diff := sqr(SaltLayer1-SaltRoot1M[i]);
          SumDiffSq1:=SumDiffSq1+Diff;
        end;
        AveSalt:=(InitSaltR+SaltLayer1)/2;
//Salinity 2nd Layer
        InitSalt2:=SaltLayer2;
        SaltIn:=(LeachEff1*Perc1[i,j]*AveSalt+Cap3[i,j]*SaltCap2)/WaterDepth2;
        SaltOut:=0.5*LeachEff2*(Perc2[i,j]+Cap2[i,j])*SaltLayer2/WaterDepth2;
        Divider:=1+0.5*LeachEff2*(Perc2[i,j]+Cap2[i,j])/WaterDepth2;
        if Divider<0.001 then Divider:=0.001;
        SaltLayer2:=(SaltLayer2+SaltIn-SaltOut)/Divider;
        if SaltLayer2<=0 then SaltLayer2:=0.1;
        if (SaltRoot2M[i]>0) and (j=Steplength) then
        begin
          Diff := sqr(SaltLayer2-SaltRoot2M[i]);
          SumDiffSq2:=SumDiffSq2+Diff;
        end;
        SaltCap:=SaltLayer2;
        AveSalt:=(InitSalt2+SaltLayer2)/2;
        SaltCap:=SaltLayer2;
//Salinity 3rd layer
        InitSalt3:=SaltLayer3;
        SaltIn:=(LeachEff2*Perc2[i,j]*AveSalt+Cap4[i,j]*SaltCap3)/WaterDepth3;
        SaltOut:=0.5*LeachEff3*(Perc3[i,j]+Cap3[i,j])*SaltLayer3/WaterDepth3;
        Divider:=1+0.5*LeachEff3*(Perc3[i,j]+Cap3[i,j])/WaterDepth3;
        if Divider<0.001 then Divider:=0.001;
        SaltLayer3:=(SaltLayer3+SaltIn-SaltOut)/Divider;
        if SaltLayer3<0 then SaltLayer3:=0.1;
        if (SaltRoot3M[i]>0) and (j=Steplength) then
        begin
          Diff := sqr(SaltLayer3-SaltRoot3M[i]);
          SumDiffSq3:=SumDiffSq3+Diff;
        end;
        SaltCap2:=SaltLayer3;
        AveSalt:=(InitSalt3+SaltLayer3)/2;
//Salinity transition zone
        InitSaltT:=SaltTrans;
        SaltIn:=(LeachEff2*Perc2[i,j]*AveSalt+GrwtUp*SaltAquif)/WaterDepthTR;
        SaltOut:=
             0.5*LeachEffTR*(DRCf[i]*SaltTrans)/WaterDepthTR;
        Divider:=1+0.5*(DRCf[i]*LeachEffTR)
                 /WaterDepthTR;
        if Divider<0.001 then Divider:=0.001;
        SaltTrans:=(SaltTrans+SaltIn-SaltOut)/Divider;
        if SaltTrans<0 then SaltTrans:=0.1;
        if (SaltTransM[i]>0) and (j=Steplength) then
        begin
          Diff := sqr(SaltTrans-SaltTransM[i]);
          SumDiffSq4:=SumDiffSq4+Diff;
        end;
        InitSaltT:=SaltTrans;
        SaltCap3:=SaltTrans;
//Salinity aquifer
        SaltIn:=(LeachEffTR*GrwtDown*AveSalt+GrwtInf*SalInf)/WaterDepthAQ;
        SaltOut:=(0.5*(GrwtOut+GrwtUp)*LeachEffAQ*SaltAquif)/WaterDepthAQ;
        Divider:= 1+0.5*(GrwtOut+GrwtUp)*LeachEffAQ/WaterDepthAQ;
        if Divider<0.001 then Divider:=0.001;
        SaltAquif:=(SaltAquif+SaltIn-SaltOut)/Divider;
        if SaltAquif<0 then SaltAquif:=0.1;
        if (SaltAquifM[i]>0) and (j=Steplength) then
        begin
          Diff := sqr(SaltAquif-SaltAquifM[i]);
          SumDiffSq5:=SumDiffSq5+Diff;
        end;
      end; {if ThreeLayer then}

    end; {for j:=1 to StepLength do}

//For output
    SaltLay1[i]:=SaltLayer1;
    SaltLay2[i]:=SaltLayer2;
    SaltLay3[i]:=SaltLayer3;
    SaltTR[i]:=SaltTrans;
    SaltAQ[i]:=SaltAquif;

  end; {for i:=1 to NrOfSteps do}

//    if OneLayer then
//    begin
      if Count1>3 then
      begin
        SumSalt:=0;
        for i:=1 to NrOfSteps do if SaltRoot1M[i]>0 then
            SumSalt:=SumSalt+SaltRoot1M[i];
        Ave:=SumSalt/Count1;
        SumAvSq:=0;
        for i:=1 to NrOfSteps do
        begin
          if SaltRoot1M[i]>0 then
             SumAvSq:=SumAvSq+Sqr(SaltRoot1M[i]);
        end;
//        Expl1:=1-sqrt(SumDiffSq/SumAvSq);
        Expl1:=1-sqrt(SumDiffSq1/SumAvSq);
      end;
//    end;
    if TwoLayer or Threelayer then
    begin
      if Count2>0 then
      begin
        SumSalt:=0;
        for i:=1 to NrOfSteps do if SaltRoot2M[i]>0 then
            SumSalt:=SumSalt+SaltRoot2M[i];
        Ave:=SumSalt/Count1;
        SumAvSq:=0;
        for i:=1 to NrOfSteps do
        begin
          if SaltRoot2M[i]>0 then
             SumAvSq:=SumAvSq+Sqr(SaltRoot2M[i]);
        end;
        SumDiffSq:=0;
//        Expl2:=1-sqrt(SumDiffSq/SumAvSq);
        Expl2:=1-sqrt(SumDiffSq2/SumAvSq);
      end;
    end;
    if ThreeLayer then
    begin
      if Count1>3 then
      begin
        SumSalt:=0;
        for i:=1 to NrOfSteps do if SaltRoot3M[i]>0 then
            SumSalt:=SumSalt+SaltRoot3M[i];
        Ave:=SumSalt/Count3;
        SumAvSq:=0;
        for i:=1 to NrOfSteps do
        begin
          if SaltRoot3M[i]>0 then
             SumAvSq:=SumAvSq+Sqr(SaltRoot3M[i]);
        end;
        SumDiffSq:=0;
        Expl3:=1-sqrt(SumDiffSq3/SumAvSq);
        if Expl1<0 then Expl3:=0;
      end;
    end;

      if Count4>3 then
      begin
        SumSalt:=0;
        for i:=1 to NrOfSteps do if SaltTransM[i]>0 then
            SumSalt:=SumSalt+SaltTransM[i];
        Ave:=SumSalt/Count4;
        SumAvSq:=0;
        for i:=1 to NrOfSteps do
        begin
          if SaltTransM[i]>0 then
             SumAvSq:=SumAvSq+Sqr(SaltTransM[i]);
        end;
        ExplTR:=1-sqrt(SumDiffSq4/SumAvSq);
        if ExplTR<0 then ExplTR:=0;
      end;
      if Count5>3 then
      begin
        SumSalt:=0;
        for i:=1 to NrOfSteps do if SaltAquifM[i]>0 then
            SumSalt:=SumSalt+SaltAquifM[i];
        Ave:=SumSalt/Count5;
        SumAvSq:=0;
        for i:=1 to NrOfSteps do
        begin
          if SaltAquifM[i]>0 then
             SumAvSq:=SumAvSq+Sqr(SaltAquifM[i]);
        end;
        ExplAQ:=1-sqrt(SumDiffSq5/SumAvSq);
        if ExplAQ<0 then ExplAQ:=0;
      end;

      NoCount1:=false;
      if Count1<4 then
      begin
        Change1:=false;
        Expl1:=0;
        NoCount1:=true;
      end;
      NoCount2:=false;
      if Count2<4 then
      begin
        Change2:=false;
        Expl2:=0;
        NoCount4:=true;
      end;
      NoCount3:=false;
      if Count3<4 then
      begin
        Change3:=false;
        Expl3:=0;
        NoCount3:=true;
      end;
      NoCount4:=false;
      if Count4<4 then
      begin
        Change4:=false;
        ExplTR:=0;
        NoCount4:=true;
      end;
      NoCount5:=false;
      if Count5<4 then
      begin
        Change5:=false;
        ExplAQ:=0;
        NoCount5:=true;
      end;
//    end;

end; {procedure SaltBalance}



procedure WriteOutput;
var FileLine : string;
    DatFile : textfile;
    OutFile : string;
begin
  if CalcDone then OutFile:=changefileext(FileName,'.out')
     else OutFile:=changefileext(InFileName,'.out');
  Assignfile(DatFile,OutFile);
  {$I-} Reset(DatFile); {$I+}
  MainForm.OutputMemo.Lines.Clear;
  while not eof (DatFile) do
  begin
    readln (DatFile,FileLine);
    MainForm.OutputMemo.Lines.Add(FileLine);
  end;
  closefile (DatFile);
end; {procedure WriteOutput}


procedure WriteSaltTable;
var i : integer;
    AuxStr1, AuxStr2, AuxStr3, AuxStr4, AuxStr5,
    AuxStr6, AuxStr7, AuxStr8, AuxStr9, AuxStr10 : string;
begin
  MainForm.OutGrid.RowCount:=NrOfSteps+1;
  MainForm.OutGrid.Cells[0,0]:='Time step';
  MainForm.OutGrid.Cells[1,0]:='dS/m';
  MainForm.OutGrid.Cells[2,0]:='dS/m';
  MainForm.OutGrid.Cells[3,0]:='dS/m';
  MainForm.OutGrid.Cells[4,0]:='dS/m';
  MainForm.OutGrid.Cells[5,0]:='dS/m';
  MainForm.OutGrid.Cells[6,0]:='dS/m';
  MainForm.OutGrid.Cells[7,0]:='dS/m';
  MainForm.OutGrid.Cells[8,0]:='dS/m';
  MainForm.OutGrid.Cells[9,0]:='dS/m';
  MainForm.OutGrid.Cells[10,0]:='dS/m';

  if OneLayer or (NrOfLayers=1) then
  begin
    MainForm.OutGrid.Left:=100;
    MainForm.OutGrid.ColCount:=7;
    MainForm.OutGrid.Width:=480;
    MainForm.ClipLabel.Left:=106;
    MainForm.ClipButton.Left:=102;
  end;
  if TwoLayer or (NrOfLayers=2) then
  begin
    MainForm.OutGrid.Left:=50;
    MainForm.OutGrid.ColCount:=9;
    MainForm.OutGrid.Width:=610;
    MainForm.ClipLabel.Left:=56;
    MainForm.ClipButton.Left:=52;
  end;
  if ThreeLayer or (NrOfLayers=3) then
  begin
    MainForm.OutGrid.Left:=10;
    MainForm.OutGrid.ColCount:=11;
    MainForm.OutGrid.Width:=740;
    MainForm.ClipLabel.Left:=16;
    MainForm.ClipButton.Left:=12;
  end;
  MainForm.ClipLabel.Visible:=true;
  MainForm.ClipButton.Visible:=true;;

  for i:=1 to NrOfSteps do MainForm.OutGrid.Cells[0,i]:=inttostr(i);

  if OneLayer or (NrOfLayers=1) then
  begin
    for i:=1 to NrOfSteps do
    begin
      AuxStr1:=format('%13.2f',[SaltLay1[i]]);
      if SaltRoot1M[i]=-1 then AuxStr2:='            -'
      else
        AuxStr2:=format('%13.2f',[SaltRoot1M[i]]);
      AuxStr3:=format('%9.2f',[SaltTR[i]]);
      if SaltTransM[i]=-1 then AuxStr4:='            -'
      else
        AuxStr4:=format('%9.2f',[SaltTransM[i]]);
      AuxStr5:=format('%9.2f',[SaltAQ[i]]);
      if SaltAquifM[i]=-1 then AuxStr6:='            -'
      else
       AuxStr6:=format('%9.2f',[SaltAquifM[i]]);
      MainForm.OutGrid.Cells[1,i]:=AuxStr1;
      MainForm.OutGrid.Cells[2,i]:=AuxStr2;
      MainForm.OutGrid.Cells[3,i]:=AuxStr3;
      MainForm.OutGrid.Cells[4,i]:=AuxStr4;
      MainForm.OutGrid.Cells[5,i]:=AuxStr5;
      MainForm.OutGrid.Cells[6,i]:=AuxStr6;
    end; {for i:=1 to NrOfSteps do}
    MainForm.OutSalt1Memo.Visible:=true;
    MainForm.HydrOutMemo.Visible:=false;
 end; {if OneLayer or (NrOfLayers=1) then}

  if TwoLayer or (NrOfLayers=2) then
  begin
    for i:=1 to NrOfSteps do
    begin
      AuxStr1:=format('%13.2f',[SaltLay1[i]]);
      if SaltRoot1M[i]=-1 then AuxStr2:='            -'
      else
        AuxStr2:=format('%13.2f',[SaltRoot1M[i]]);
      AuxStr3:=format('%13.2f',[SaltLay2[i]]);
      if SaltRoot2M[i]=-1 then AuxStr4:='            -'
      else
        AuxStr4:=format('%11.2f',[SaltRoot2M[i]]);
      AuxStr5:=format('%9.2f',[SaltTR[i]]);
      if SaltTransM[i]=-1 then AuxStr6:='            -'
      else
        AuxStr6:=format('%9.2f',[SaltTransM[i]]);
      AuxStr7:=format('%9.2f',[SaltAQ[i]]);
      if SaltAquifM[i]=-1 then AuxStr9:='            -'
      else
       AuxStr8:=format('%9.2f',[SaltAquifM[i]]);
      MainForm.OutGrid.Cells[1,i]:=AuxStr1;
      MainForm.OutGrid.Cells[2,i]:=AuxStr2;
      MainForm.OutGrid.Cells[3,i]:=AuxStr3;
      MainForm.OutGrid.Cells[4,i]:=AuxStr4;
      MainForm.OutGrid.Cells[5,i]:=AuxStr5;
      MainForm.OutGrid.Cells[6,i]:=AuxStr6;
      MainForm.OutGrid.Cells[7,i]:=AuxStr7;
      MainForm.OutGrid.Cells[8,i]:=AuxStr8;
    end; {for i:=1 to NrOfSteps do}
    MainForm.OutSalt2Memo.Visible:=true;
    MainForm.HydrOutMemo.Visible:=false;
 end; {if TwoLayer or (NrOfLayers=2) then}


  if ThreeLayer or (NrOfLayers=3) then
  begin
    for i:=1 to NrOfSteps do
    begin
      AuxStr1:=format('%13.2f',[SaltLay1[i]]);
      if SaltRoot1M[i]=-1 then AuxStr2:='            -'
      else
        AuxStr2:=format('%13.2f',[SaltRoot1M[i]]);
      AuxStr3:=format('%13.2f',[SaltLay2[i]]);
      if SaltRoot2M[i]=-1 then AuxStr4:='            -'
      else
        AuxStr4:=format('%11.2f',[SaltRoot2M[i]]);
      AuxStr5:=format('%13.2f',[SaltLay3[i]]);
      if SaltRoot3M[i]=-1 then AuxStr6:='            -'
      else
        AuxStr6:=format('%9.2f',[SaltRoot3M[i]]);
      AuxStr7:=format('%9.2f',[SaltTR[i]]);
      if SaltTransM[i]=-1 then AuxStr8:='            -'
      else
        AuxStr8:=format('%9.2f',[SaltTransM[i]]);
      AuxStr9:=format('%9.2f',[SaltAQ[i]]);
      if SaltAquifM[i]=-1 then AuxStr10:='            -'
      else
       AuxStr10:=format('%9.2f',[SaltAquifM[i]]);
      MainForm.OutGrid.Cells[1,i]:=AuxStr1;
      MainForm.OutGrid.Cells[2,i]:=AuxStr2;
      MainForm.OutGrid.Cells[3,i]:=AuxStr3;
      MainForm.OutGrid.Cells[4,i]:=AuxStr4;
      MainForm.OutGrid.Cells[5,i]:=AuxStr5;
      MainForm.OutGrid.Cells[6,i]:=AuxStr6;
      MainForm.OutGrid.Cells[7,i]:=AuxStr7;
      MainForm.OutGrid.Cells[8,i]:=AuxStr8;
      MainForm.OutGrid.Cells[9,i]:=AuxStr9;
      MainForm.OutGrid.Cells[10,i]:=AuxStr10;
    end; {for i:=1 to NrOfSteps do}
    MainForm.HydrOutMemo.Visible:=false;
    MainForm.OutSalt3Memo.Visible:=true;
  end; {if ThreeLayer or (NrOfLayers=3) then}

end; {procedure WriteSaltTable}



procedure WriteHydrTable;
var i : integer;
begin
  MainForm.OutGrid.RowCount:=NrOfSteps+1;
  MainForm.OutGrid.ColCount:=9;
  MainForm.OutGrid.Width:=612;
  MainForm.OutGrid.Left:=50;
  MainForm.ClipLabel.Left:=56;
  MainForm.ClipButton.Left:=52;
  MainForm.ClipLabel.Visible:=true;
  MainForm.ClipButton.Visible:=true;;
  MainForm.OutGrid.Cells[0,0]:='Time step';
  MainForm.OutGrid.Cells[1,0]:='mm';
  MainForm.OutGrid.Cells[2,0]:='mm';
  MainForm.OutGrid.Cells[3,0]:='mm';
  MainForm.OutGrid.Cells[4,0]:='mm';
  MainForm.OutGrid.Cells[5,0]:='mm';
  MainForm.OutGrid.Cells[6,0]:='mm';
  MainForm.OutGrid.Cells[7,0]:='mm';
  MainForm.OutGrid.Cells[8,0]:=' m';
  for i:=1 to NrOfSteps do MainForm.OutGrid.Cells[0,i]:=inttostr(i);
  for i:=1 to NrOfSteps do
  begin
    MainForm.OutGrid.Cells[1,i]:=floattostr(Rain[i]);
    MainForm.OutGrid.Cells[2,i]:=floattostr(Irri[i]);
    MainForm.OutGrid.Cells[3,i]:=floattostr(Evap[i]);
    MainForm.OutGrid.Cells[4,i]:=floattostr(PercF[i]);
    MainForm.OutGrid.Cells[5,i]:=floattostr(CapF[i]);
    MainForm.OutGrid.Cells[6,i]:=floattostr(EPAf[i]);
    MainForm.OutGrid.Cells[7,i]:=floattostr(DRCf[i]);
    MainForm.OutGrid.Cells[8,i]:=floattostr(DepthWT[i]);
  end;
  MainForm.HydrOutMemo.Visible:=true;
  MainForm.OutSalt1Memo.Visible:=false;
  MainForm.OutSalt2Memo.Visible:=false;
  MainForm.OutSalt3Memo.Visible:=false;
end; {procedure WriteHydrTable}



procedure ReadOutData;
var i : integer;
    HlpStr : string[4];
    NrStr : string[10];
    TxtStr : string[32];
    AuxStr : string[43];
    Str8A,Str8B, Str8C : string[8];
    Str9A, Str9B, Str9C, Str9D : string[9];
    Str10A, Str10B, Str10C, Str10D, Str10E, Str10G : string[10];
    Str12 : string[12];
    Str22 : string[22];
begin
  Assignfile(OutFile,OutFileName);
  {$I-} Reset(OutFile); {$I+}
  if ioresult <> 0 then
  begin
    Showmessage('The file ' + OutFileName + 'could not be opened. It ' +
                'may be damaged or in use by another program.');
    closefile(OutFile);
    exit;
  end;
  for i:=1 to 5 do readln (OutFile,NrStr);
  readln (OutFile,NrOfLayers,NrOfSteps);
  read (OutFile,NrStr);
  if NrStr=' Drainage ' then
  begin
    readln (OutFile,Str22,DrainQH1);
    read (OutFile,TxtStr);
    read (OutFile,LEff1);
    read (OutFile,HlpStr);
  end else
  begin
    read (OutFile,Str22,LEff1);
    read (OutFile,HlpStr);
  end;
  if HlpStr='  (o' then
  begin
    Optim1:=true;
    read (OutFile,AuxStr);
    readln (OutFile,Expl1);
  end else
  begin
    Optim1:=false;
    read (OutFile,AuxStr);
    readln (OutFile,HlpStr);
    if HlpStr<>' n.a' then
     Expl1:=stringtofloat (HlpStr);
  end;
  if NrOfLayers>1 then
  begin
    read (OutFile,TxtStr);
    read (OutFile,LEff2);
    read (OutFile,HlpStr);
    if HlpStr='  (o' then
    begin
      Optim2:=true;
      read (OutFile,AuxStr);
      readln (OutFile,Expl2);
    end else
    begin
      Optim2:=false;
      read (OutFile,AuxStr);
      readln (OutFile,HlpStr);
      if HlpStr<>' n.a' then
      Expl2:=stringtofloat (HlpStr);
    end;
  end;
  if NrOfLayers>2 then
  begin
    read (OutFile,TxtStr);
    read (OutFile,LEff3);
    read (OutFile,HlpStr);
    if HlpStr='  (o' then
    begin
      Optim3:=true;
      read (OutFile,AuxStr);
      readln (OutFile,Expl3);
    end else
    begin
      Optim3:=false;
      read (OutFile,AuxStr);
      readln (OutFile,HlpStr);
      if HlpStr<>' n.a' then
      Expl3:=stringtofloat (HlpStr);
    end;
  end;
  read (OutFile,TxtStr);
  read (OutFile,LEff4);
  read (OutFile,HlpStr);
  if HlpStr='  (o' then
  begin
    Optim4:=true;
    read (OutFile,AuxStr);
    readln (OutFile,ExplTR);
  end else
  begin
    Optim4:=false;
    read (OutFile,AuxStr);
      readln (OutFile,HlpStr);
      if HlpStr<>' n.a' then
      ExplTR:=stringtofloat (HlpStr);
  end;
  read (OutFile,TxtStr);
  read (OutFile,LEff5);
  read (OutFile,HlpStr);
  if HlpStr='  (o' then
  begin
    Optim5:=true;
    read (OutFile,AuxStr);
    readln (OutFile,ExplAQ);
  end else
  begin
    Optim5:=false;
    read (OutFile,AuxStr);
    readln (OutFile,HlpStr);
    if HlpStr<>' n.a' then
    ExplAQ:=stringtofloat (HlpStr);
  end;

  setlength (PercF,NrOfSteps+1);
  setlength (CapF,NrOfSteps+1);
  setlength (EPAf,NrOfSteps+1);
  setlength (DRCf,NrOfSteps+1);
  setlength (DepthWT,NrOfSteps+1);
  setlength (SaltLay1,NrOfSteps+1);
  setlength (SaltLay2,NrOfSteps+1);
  setlength (SaltLay3,NrOfSteps+1);
  setlength (SaltTR,NrOfSteps+1);
  setlength (SaltAQ,NrOfSteps+1);
  HlpStr:='';
  while not (HlpStr=' - -') do readln (OutFile,HlpStr);
  for i:=1 to NrOfSteps do
  begin
    if NrOfLayers=1 then
    begin
       readln (OutFile,NrStr,Str10A,Str10B,Str9A,Str9B,Str10C,Str10D,Str12,
               Str10E);
       PercF[i]:=StringToFloat(Str10A);
       CapF[i]:=StringToFloat(Str10B);
       EPAf[i]:=StringToFloat(Str9A);
       DRCf[i]:=StringToFloat(Str9B);
       DepthWT[i]:=StringToFloat(Str10C);
       SaltLay1[i]:=StringToFloat(Str10D);
       SaltTR[i]:=StringToFloat(Str12);
       SaltAQ[i]:=StringToFloat(Str10E);
    end;

    if NrOfLayers=2 then
    begin
       readln (OutFile,NrStr,Str10A,Str10B,Str10G,Str9A,Str9B,Str8A,Str8B,Str9C,
               Str10C);
       PercF[i]:=StringToFloat(Str10A);
       CapF[i]:=StringToFloat(Str10B);
       EPAf[i]:=StringToFloat(Str10G);
       DRCf[i]:=StringToFloat(Str9A);
       DepthWT[i]:=StringToFloat(Str9B);
       SaltLay1[i]:=StringToFloat(Str8A);
       SaltLay2[i]:=StringToFloat(Str8B);
       SaltTR[i]:=StringToFloat(Str9C);
       SaltAQ[i]:=StringToFloat(Str10C);
    end;
    if NrOfLayers=3 then
    begin
       readln (OutFile,NrStr,Str10A,Str10B,Str10G,Str10C,Str8C,Str8A,Str9C,
               Str9D,Str9B,Str8B);
       PercF[i]:=StringToFloat(Str10A);
       CapF[i]:=StringToFloat(Str10B);
       EPAf[i]:=StringToFloat(Str10G);
       DRCf[i]:=StringToFloat(Str10C);
       DepthWT[i]:=StringToFloat(Str8C);
       SaltLay1[i]:=StringToFloat(Str8A);
       SaltLay2[i]:=StringToFloat(Str9C);
       SaltLay3[i]:=StringToFloat(Str9D);
       SaltTR[i]:=StringToFloat(Str9B);
       SaltAQ[i]:=StringToFloat(Str8B);
    end;
  end;
  closefile (OutFile);
end; {procedure ReadOutData}



procedure ReadInpData;
var i : integer;
    Str12  : string[12];
    Str8   : string[8];
    Str9   : string[9];
    Str10  : string[10];
    Str18  : string[18];
    Str20  : string[20];
    Str28  : string[28];
    InFile : textfile;
    TxtA,TxtB,TxtC : string[15];
    NewVersion: boolean;
begin
  Assignfile(InFile,InFileName);
  {$I-} Reset(InFile); {$I+}
  if ioresult <> 0 then
  begin
    Showmessage('The file ' + InFileName + 'could not be opened. It ' +
                'may be damaged or in use by another program.');
    closefile(InFile);
    exit;
  end;
  setlength (Rain,NrOfSteps+1);
  setlength (Irri,NrOfSteps+1);
  setlength (Irrig,NrOfSteps+1);
  setlength (Evap,NrOfSteps+1);
  setlength (Stor,NrOfSteps+1);
  setlength (StorTot,NrOfSteps+1);
  for i:=1 to 2 do readln (InFile,Str28);
  readln (InFile,Str28,Title1);
  readln (InFile,Str28,Title2);
  for i:=1 to 3 do readln (InFile,Str8);
  readln (InFile,Str28,NrOfLayers);
  for i:=1 to 5 do readln (Infile,Str8);
  readln(InFile, Str20);
  if Str20='    Initial data' then NewVersion := false
  else NewVersion := true;
  OneLayer:=false;
  TwoLayer:=false;
  ThreeLayer:=false;
  if NrOfLayers=1 then OneLayer:=true;
  if NrOfLayers=2 then TwoLayer:=true;
  if NrOfLayers=3 then ThreeLayer:=true;
  if NewVersion then
  begin
    if OneLayer then for i:=1 to 8 do readln (InFile,Str10);
    if TwoLayer then for i:=1 to 9 do readln (InFile,Str10);
    if ThreeLayer then for i:=1 to 10 do readln (InFile,Str10);
    if Str10=' Initial s' then for i:=1 to 8 do readln (InFile,Str10)
    else for i:=1 to 7 do readln (InFile,Str10);
  end else
    for i:=1 to 15 do readln (InFile,Str10);

  for i:=1 to NrOfSteps do
  begin
//      readln (InFile,Str12,Rain[i],Irri[i],Evap[i]);
    readln (InFile,Str12,TxtA,TxtB,TxtC);
    Rain[i]:=stringtofloat(TxtA);
    Irri[i]:=stringtofloat(TxtB);
    Evap[i]:=stringtofloat(TxtC);
  end;

  if NewVersion then
  begin
    if Onelayer then for i:=1 to 15 do readln (Infile,Str12);
    if Twolayer then for i:=1 to 21 do readln (Infile,Str12);
    if Threelayer then for i:=1 to 22 do readln (Infile,Str12);
  end else
  begin
    if Onelayer then for i:=1 to 14 do readln (Infile,Str12);
    if Twolayer then for i:=1 to 20 do readln (Infile,Str12);
    if Threelayer then for i:=1 to 21 do readln (Infile,Str12);
  end;

  setlength (SaltRoot1M,NrOfSteps+1);
  setlength (SaltRoot2M,NrOfSteps+1);
  setlength (SaltRoot3M,NrOfSteps+1);
  setlength (SaltTransM,NrOfSteps+1);
  setlength (SaltAquifM,NrOfSteps+1);
  if OneLayer or (NrOfLayers=1) then
  begin
    for i:=1 to NrOfSteps do
    begin
      read (InFile,Str10);
      read (Infile,Str12);
      while pos (' ',Str12)>0 do delete(Str12,Pos(' ',Str12),1);
      if Str12='-' then SaltRoot1M[i]:=-1
         else SaltRoot1M[i]:=stringtofloat(Str12);
      read (Infile,Str18);
      while pos (' ',Str18)>0 do delete(Str18,Pos(' ',Str18),1);
      if Str18='-' then SaltTransM[i]:=-1
         else SaltTransM[i]:=stringtofloat(Str18);
      readln (Infile,Str10);
      while pos (' ',Str10)>0 do delete(Str10,Pos(' ',Str10),1);
      if Str10='-' then SaltAquifM[i]:=-1
         else SaltAquifM[i]:=stringtofloat(Str10);
    end;
  end;
  if TwoLayer or (NrOfLayers=2) then
  begin
    for i:=1 to NrOfSteps do
    begin
      read (InFile,Str10);
      read (Infile,Str8);
      while pos (' ',Str8)>0 do delete(Str8,Pos(' ',Str8),1);
      if Str8='-' then SaltRoot1M[i]:=-1
         else SaltRoot1M[i]:=stringtofloat(Str8);
      read (Infile,Str9);
      while pos (' ',Str9)>0 do delete(Str9,Pos(' ',Str9),1);
      if Str9='-' then SaltRoot2M[i]:=-1
         else SaltRoot2M[i]:=stringtofloat(Str9);
      read (Infile,Str18);
      while pos (' ',Str18)>0 do delete(Str18,Pos(' ',Str18),1);
      if Str18='-' then SaltTransM[i]:=-1
         else SaltTransM[i]:=stringtofloat(Str18);
      readln (Infile,Str10);
      while pos (' ',Str10)>0 do delete(Str10,Pos(' ',Str10),1);
      if Str10='-' then SaltAquifM[i]:=-1
         else SaltAquifM[i]:=stringtofloat(Str10);
    end;
  end;
  if ThreeLayer or (NrOfLayers=3) then
  begin
    for i:=1 to NrOfSteps do
    begin
      read (InFile,Str10);
      read (Infile,Str8);
      while pos (' ',Str8)>0 do delete(Str8,Pos(' ',Str8),1);
      if Str8='-' then SaltRoot1M[i]:=-1
         else SaltRoot1M[i]:=stringtofloat(Str8);
      read (Infile,Str8);
      while pos (' ',Str8)>0 do delete(Str8,Pos(' ',Str8),1);
      if Str8='-' then SaltRoot2M[i]:=-1
         else SaltRoot2M[i]:=stringtofloat(Str8);
      read (Infile,Str8);
      while pos (' ',Str8)>0 do delete(Str8,Pos(' ',Str8),1);
      if Str8='-' then SaltRoot3M[i]:=-1
         else SaltRoot3M[i]:=stringtofloat(Str8);
      read (Infile,Str18);
      while pos (' ',Str18)>0 do delete(Str18,Pos(' ',Str18),1);
      if Str18='-' then SaltTransM[i]:=-1
         else SaltTransM[i]:=stringtofloat(Str18);
      readln (Infile,Str10);
      while pos (' ',Str10)>0 do delete(Str10,Pos(' ',Str10),1);
      if Str10='-' then SaltAquifM[i]:=-1
         else SaltAquifM[i]:=stringtofloat(Str10);
    end;
  end;
 if SaltRoot1M[1]>0 then SaltLayer1i:=SaltRoot1M[1];
 if SaltRoot2M[1]>0 then SaltLayer2i:=SaltRoot2M[1];
 if SaltRoot3M[1]>0 then SaltLayer3i:=SaltRoot3M[1];

  closefile (InFile);
end; {procedure ReadInpData}


end.


